/* Scrutari - Copyright (c) 2009-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.explore;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.servlets.ScrutariRequestMap;
import fr.exemole.sctserver.servlets.explore.htmlpages.SupermotcleCroisementPage;
import fr.exemole.sctserver.servlets.explore.htmlpages.SupermotcleListePage;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.XmlResponseHandler;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.xml.XmlProducer;


/**
 *
 * @author Vincent Calame
 */
public class ExploreRequest implements ExploreConstants {

    private static final String SUPERMOTCLE_PATH = "/supermotcle/";
    private static final String LISTS_PATH = "/lists/";
    private ScrutariRequestMap requestMap;
    private boolean isInit;
    private short responseHandlerType;
    private SctServer sctServer;
    private int moisLog;
    private int pageLangInteger;
    private SearchSequence searchSequence;

    private ExploreRequest(SctServer sctServer, String pathInfo, ScrutariRequestMap requestMap) {
        this.requestMap = requestMap;
        this.sctServer = sctServer;
        initRequestMap();
        isInit = false;
        if (pathInfo.equals("/")) {
        } else if (pathInfo.startsWith(SUPERMOTCLE_PATH)) {
            isInit = checkSupermotcleRequest(pathInfo.substring(SUPERMOTCLE_PATH.length()));
        } else if (pathInfo.startsWith(LISTS_PATH)) {
            isInit = checkListRequest(pathInfo.substring(LISTS_PATH.length()));
        }
    }

    private void initRequestMap() {
        searchSequence = RequestMapUtils.getSearchSequence(requestMap);
    }

    public static ExploreRequest init(SctContext sctContext, String pathInfo, ScrutariRequestMap requestMap) {
        ExploreRequest exploreRequest = new ExploreRequest(sctContext.getSctServer(), pathInfo, requestMap);
        if (exploreRequest.isInit) {
            return exploreRequest;
        } else {
            return null;
        }
    }

    public ResponseHandler getResponseHandler() {
        switch (responseHandlerType) {
            case SUPERMOTCLE_LISTE_PAGE:
                return new SupermotcleListePage(sctServer, pageLangInteger, searchSequence);
            case SUPERMOTCLE_CROISEMENT_PAGE:
                return new SupermotcleCroisementPage(sctServer, pageLangInteger);
            case BASE_LIST_XMLDATA:
            case LOG_LIST_XMLDATA:
            case MONTH_LOG_XMLDATA:
                return getXmlResponseHandler();
            default:
                throw new SwitchException("responseHandlerType = " + responseHandlerType);
        }
    }

    private ResponseHandler getXmlResponseHandler() {
        XmlProducer xmlProducer = null;
        switch (responseHandlerType) {
            case BASE_LIST_XMLDATA:
                xmlProducer = ExploreXmlProducerFactory.getBaseListXmlProducer(sctServer);
                break;
            case LOG_LIST_XMLDATA:
                xmlProducer = ExploreXmlProducerFactory.getLogListXmlProducer(sctServer);
                break;
            case MONTH_LOG_XMLDATA:
                xmlProducer = ExploreXmlProducerFactory.getMonthLogXmlProducer(sctServer, moisLog);
                break;
        }
        XmlResponseHandler xmlResponseHandler = new XmlResponseHandler(xmlProducer);
        return xmlResponseHandler;
    }

    private boolean checkSupermotcleRequest(String fileName) {
        Page page = getPage(fileName);
        if (page == null) {
            return false;
        }
        String commandName = page.getCommandName();
        if (commandName.startsWith("liste")) {
            if (page.hasLangInteger()) {
                responseHandlerType = SUPERMOTCLE_LISTE_PAGE;
                return true;
            } else {
            }
        } else if (commandName.startsWith("croisement")) {
            if (page.hasLangInteger()) {
                responseHandlerType = SUPERMOTCLE_CROISEMENT_PAGE;
                return true;
            } else {
            }
        }
        return false;
    }

    private boolean checkListRequest(String fileName) {
        Page page = getPage(fileName);
        if (page == null) {
            return false;
        }
        if (!page.getExtension().equals("xml")) {
            return false;
        }
        String commandName = page.getCommandName();
        if (commandName.equals("base-list")) {
            responseHandlerType = BASE_LIST_XMLDATA;
            return true;
        }
        if (commandName.equals("log-list")) {
            responseHandlerType = LOG_LIST_XMLDATA;
            return true;
        }
        if (commandName.equals("log")) {
            int dateInteger = page.getDateInteger();
            if (DateInteger.getDateIntegerType(dateInteger) == DateInteger.TYPE_MOIS) {
                responseHandlerType = MONTH_LOG_XMLDATA;
                moisLog = dateInteger;
                return true;
            } else {
                return false;
            }
        }
        return false;

    }

    private Page getPage(String fileName) {
        Page page = new Page();
        if (!page.init(fileName)) {
            return null;
        }
        boolean withLangInteger = page.hasLangInteger();
        if (withLangInteger) {
            pageLangInteger = page.getLangInteger();
        }
        return page;
    }


    private class Page {

        private String extension;
        private int langInteger = 0;
        private int dateInteger = 0;
        private String commandName;

        Page() {
        }

        boolean init(String fileName) {
            int idx = fileName.lastIndexOf('.');
            if ((idx == -1) || (idx == (fileName.length() - 1))) {
                return false;
            }
            extension = fileName.substring(idx + 1);
            commandName = fileName.substring(0, idx);
            int idx2 = commandName.lastIndexOf('_');
            if (idx2 != -1) {
                String complement = commandName.substring(idx2 + 1);
                try {
                    langInteger = LangInteger.parse(complement);
                } catch (LangIntegerException lie) {
                    try {
                        dateInteger = DateInteger.parse(complement);
                    } catch (DateIntegerException ide) {
                        return false;
                    }
                }
                commandName = commandName.substring(0, idx2);
            }
            return true;
        }

        public String getExtension() {
            return extension;
        }

        public int getLangInteger() {
            return langInteger;
        }

        public int getDateInteger() {
            return dateInteger;
        }

        public String getCommandName() {
            return commandName;
        }

        public boolean hasLangInteger() {
            return (langInteger != 0);
        }

    }

}
