/* ScrutariLib_DB - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import java.util.List;
import java.util.Collection;


/**
 *
 * @author Vincent Calame
 */
public class CodeListUtils {

    public final static CodeList EMPTY_CODELIST = new EmptyCodeList();
    public final static ElementCodeList EMPTY_ELEMENTCODELIST = new EmptyElementCodeList();
    public final static IndexationCodeList EMPTY_INDEXATIONCODELIST = new EmptyIndexationCodeList();

    public static CodeList toCodeList(int[] intArray) {
        if (intArray == null) {
            return EMPTY_CODELIST;
        }
        if (intArray.length == 0) {
            return EMPTY_CODELIST;
        }
        return new ArrayCodeList(intArray);
    }

    public static CodeList toCodeList(Collection<Integer> codeCollection) {
        if (codeCollection == null) {
            return EMPTY_CODELIST;
        }
        int size = codeCollection.size();
        if (size == 0) {
            return EMPTY_CODELIST;
        }
        if (size == 1) {
            return new UniqueCodeList(codeCollection.iterator().next().intValue());
        }
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : codeCollection) {
            result[p] = itg.intValue();
            p++;
        }
        return new ArrayCodeList(result);
    }

    public static ElementCodeList toElementCodeList(List<Integer> elementCodeList, List<Integer> subsetCodeList) {
        if (elementCodeList == null) {
            return EMPTY_ELEMENTCODELIST;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_ELEMENTCODELIST;
        }
        if (size == 1) {
            return new UniqueElementCodeList(elementCodeList.get(0), subsetCodeList.get(0));
        }
        int[] result = new int[size];
        int[] subsetResult = new int[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            subsetResult[i] = subsetCodeList.get(i);
        }
        return new ArrayElementCodeList(result, subsetResult);
    }

    public static IndexationCodeList toIndexatoinCodeList(List<Integer> elementCodeList, List<Float> ponderationList) {
        if (elementCodeList == null) {
            return EMPTY_INDEXATIONCODELIST;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_INDEXATIONCODELIST;
        }
        if (size == 1) {
            return new UniqueIndexationCodeList(elementCodeList.get(0), ponderationList.get(0));
        }
        int[] result = new int[size];
        float[] ponderationResult = new float[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            ponderationResult[i] = ponderationList.get(i);
        }
        return new ArrayIndexationCodeList(result, ponderationResult);
    }


    private static class EmptyCodeList implements CodeList {

        public int getCodeCount() {
            return 0;
        }

        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyElementCodeList implements ElementCodeList {

        public int getCodeCount() {
            return 0;
        }

        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        public int getSubsetCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyIndexationCodeList implements IndexationCodeList {

        public int getCodeCount() {
            return 0;
        }

        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        public float getPonderation(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayCodeList implements CodeList {

        private int[] array;

        public ArrayCodeList(int[] array) {
            this.array = array;
        }

        public int getCodeCount() {
            return array.length;
        }

        public int getCode(int i) {
            return array[i];
        }

    }


    private static class ArrayElementCodeList extends ArrayCodeList implements ElementCodeList {

        private int[] subsetCodeArray;

        public ArrayElementCodeList(int[] array, int[] subsetCodeArray) {
            super(array);
            this.subsetCodeArray = subsetCodeArray;
        }

        public int getSubsetCode(int i) {
            return subsetCodeArray[i];
        }

    }


    private static class ArrayIndexationCodeList extends ArrayCodeList implements IndexationCodeList {

        private float[] ponderationArray;

        public ArrayIndexationCodeList(int[] array, float[] ponderationArray) {
            super(array);
            this.ponderationArray = ponderationArray;
        }

        public float getPonderation(int i) {
            return ponderationArray[i];
        }

    }


    private static class UniqueCodeList implements CodeList {

        private int unique;

        public UniqueCodeList(int unique) {
            this.unique = unique;
        }

        public int getCodeCount() {
            return 1;
        }

        public int getCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return unique;
        }

    }


    private static class UniqueElementCodeList extends UniqueCodeList implements ElementCodeList {

        private int subsetCode;

        public UniqueElementCodeList(int unique, int subsetCode) {
            super(unique);
            this.subsetCode = subsetCode;
        }

        public int getSubsetCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return subsetCode;
        }

    }


    private static class UniqueIndexationCodeList extends UniqueCodeList implements IndexationCodeList {

        private float ponderation;

        public UniqueIndexationCodeList(int unique, float ponderation) {
            super(unique);
            this.ponderation = ponderation;
        }

        public float getPonderation(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return ponderation;
        }

    }

}
