/* ScrutariLib_DB - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.db.lexiebuilder.LexieOccurrencesMapBuilder;
import net.mapeadores.scrutari.db.lexiebuilder.LexieUnitBuilder;
import net.mapeadores.scrutari.lexie.FicheLexification;
import net.mapeadores.scrutari.lexie.LexieOccurrencesList;
import net.mapeadores.scrutari.lexie.MotcleLexification;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnit;
import net.mapeadores.scrutari.lexie.ScrutariLexieUnitList;
import net.mapeadores.util.text.collation.map.CollatedKeyMap;


/**
 *
 * @author Vincent Calame
 */
public class LexieAccessProviderBuilder {

    private LexieOccurrencesMapBuilder ficheLexieOccurrencesMapBuilder;
    private LexieOccurrencesMapBuilder motcleLexieOccurrencesMapBuilder;
    private LexieUnitBuilder lexieUnitBuilder;
    private LexieCacheWriter lexieCacheWriter;

    public LexieAccessProviderBuilder(AttributeKeyCoder attributeKeyCoder, LexieCacheWriter lexieCacheWriter) {
        this.lexieCacheWriter = lexieCacheWriter;
        lexieUnitBuilder = new LexieUnitBuilder();
        ficheLexieOccurrencesMapBuilder = new LexieOccurrencesMapBuilder(attributeKeyCoder, lexieUnitBuilder, true);
        motcleLexieOccurrencesMapBuilder = new LexieOccurrencesMapBuilder(attributeKeyCoder, lexieUnitBuilder, false);
    }

    public void newCorpus(int codecorpus) {
        checkLexieOccurrencesMap(ficheLexieOccurrencesMapBuilder);
        ficheLexieOccurrencesMapBuilder.setCurrentSubsetCode(codecorpus);
    }

    public void initFicheData(FicheData ficheData) {
        FicheLexification ficheLexification = ficheLexieOccurrencesMapBuilder.initFicheData(ficheData);
        lexieCacheWriter.cacheFicheLexification(ficheData.getCodeFiche(), ficheLexification);
    }

    public void newThesaurus(int codethesaurus) {
        checkLexieOccurrencesMap(motcleLexieOccurrencesMapBuilder);
        motcleLexieOccurrencesMapBuilder.setCurrentSubsetCode(codethesaurus);
    }

    public void initMotcleData(MotcleData motcleData) {
        MotcleLexification motcleLexification = motcleLexieOccurrencesMapBuilder.initMotcleData(motcleData);
        lexieCacheWriter.cacheMotcleLexification(motcleData.getCodeMotcle(), motcleLexification);
    }

    public LexieAccessProvider toLexieAccessProvider() {
        checkLexieOccurrencesMap(ficheLexieOccurrencesMapBuilder);
        checkLexieOccurrencesMap(motcleLexieOccurrencesMapBuilder);
        int lexieCount = 0;
        Collection<ScrutariLexieUnit> coll = lexieUnitBuilder.getLexieUnitList();
        Map<Integer, InternalScrutariLexieUnitList> unitByLangMap = new HashMap<Integer, InternalScrutariLexieUnitList>();
        for (ScrutariLexieUnit scrutariLexieUnit : coll) {
            int langInteger = scrutariLexieUnit.getLangInteger();
            InternalScrutariLexieUnitList scrutariLexieUnitList = unitByLangMap.get(langInteger);
            if (scrutariLexieUnitList == null) {
                scrutariLexieUnitList = new InternalScrutariLexieUnitList();
                unitByLangMap.put(langInteger, scrutariLexieUnitList);
            }
            scrutariLexieUnitList.addScrutariLexieUnit(scrutariLexieUnit);
            lexieCount++;
        }
        for (Map.Entry<Integer, InternalScrutariLexieUnitList> entry : unitByLangMap.entrySet()) {
            lexieCacheWriter.cacheScrutariLexieUnitList(entry.getKey(), entry.getValue());
        }
        InternalLexieAccessProvider internalLexieAccessProvider = new InternalLexieAccessProvider();
        internalLexieAccessProvider.lexieCacheReaderFactory = lexieCacheWriter.endCacheWrite();
        internalLexieAccessProvider.lexieCount = lexieCount;
        internalLexieAccessProvider.lexieMapByLangMap = lexieUnitBuilder.getLexieMapByLangMap();
        return internalLexieAccessProvider;
    }

    private void checkLexieOccurrencesMap(LexieOccurrencesMapBuilder lexieOccurrencesMapBuilder) {
        int currentSubsetCode = lexieOccurrencesMapBuilder.getCurrentSubsetCode();
        Map<Long, LexieOccurrencesList> map = lexieOccurrencesMapBuilder.popLexieOccurencesListMap();
        for (Map.Entry<Long, LexieOccurrencesList> entry : map.entrySet()) {
            long lg = entry.getKey();
            LexieOccurrencesList lexieOccurrencesList = entry.getValue();
            lexieCacheWriter.cacheLexieOccurrencesList(lg, currentSubsetCode, lexieOccurrencesList);
        }
    }


    private static class InternalLexieAccessProvider implements LexieAccessProvider {

        private LexieCacheReaderFactory lexieCacheReaderFactory;
        private Map<Integer, CollatedKeyMap> lexieMapByLangMap;
        private int lexieCount;

        private InternalLexieAccessProvider() {
        }

        public LexieAccess openLexieAccess() {
            return new InternalLexieAccess();
        }


        private class InternalLexieAccess implements LexieAccess {

            private LexieCacheReader lexieCacheReader;

            private InternalLexieAccess() {
            }

            public int getLexieCount() {
                return lexieCount;
            }

            public CollatedKeyMap getCollatedLexieMapByLang(int langInteger) {
                return lexieMapByLangMap.get(langInteger);
            }

            public LexieOccurrencesList getLexieOccurrencesList(long lexieId, int subsetCode) {
                testCacheReader();
                return lexieCacheReader.getLexieOccurrencesList(lexieId, subsetCode);
            }

            public ScrutariLexieUnitList getScrutariLexieUnitList(int langInteger) {
                testCacheReader();
                return lexieCacheReader.getScrutariLexieUnitList(langInteger);
            }

            public FicheLexification getFicheLexification(int codefiche) {
                testCacheReader();
                return lexieCacheReader.getFicheLexification(codefiche);
            }

            public MotcleLexification getMotcleLexification(int codemotcle) {
                testCacheReader();
                return lexieCacheReader.getMotcleLexification(codemotcle);
            }

            public void close() {
                if (lexieCacheReader != null) {
                    lexieCacheReader.close();
                }
            }

            private void testCacheReader() {
                if (lexieCacheReader == null) {
                    lexieCacheReader = lexieCacheReaderFactory.newReader();
                }
            }

        }

    }


    private static class InternalScrutariLexieUnitList implements ScrutariLexieUnitList {

        private List<ScrutariLexieUnit> list = new ArrayList<ScrutariLexieUnit>();

        public int getScrutariLexieUnitCount() {
            return list.size();
        }

        public ScrutariLexieUnit getScrutariLexieUnit(int i) {
            return list.get(i);
        }

        private void addScrutariLexieUnit(ScrutariLexieUnit scrutariLexieUnit) {
            list.add(scrutariLexieUnit);
        }

    }

}
