/* ScrutariLib_DB - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db;

import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.IndexationBuilder;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.IndexationKey;
import net.scrutari.datauri.ScrutariDataURI;


/**
 * Classe regroupant les différents élément d'une base.
 *
 * @author Vincent Calame
 */
public class ScrutariDB implements DataAccessProvider, LexieAccessProvider {

    private ScrutariDBName scrutariDBName;
    private DataAccessProvider dataAccessProvider;
    private LexieAccessProvider lexieAccessProvider;
    private AttributeKeyCoder attributeKeyCoder;
    private LangStats langStats = new LangStats();
    private CountStats countStats = new CountStats();

    private ScrutariDB(ScrutariDBName scrutariDBName) {
        this.scrutariDBName = scrutariDBName;
    }

    public ScrutariDBName getScrutariDBName() {
        return scrutariDBName;
    }

    public DataAccess openDataAccess() {
        return dataAccessProvider.openDataAccess();
    }

    public LexieAccess openLexieAccess() {
        return lexieAccessProvider.openLexieAccess();
    }

    public AttributeKeyCoder getAttributeKeyCoder() {
        return attributeKeyCoder;
    }

    public LangStats getLangStats() {
        return langStats;
    }

    public CountStats getCountStats() {
        return countStats;
    }

    public void updateStats() {
        langStats.update(this);
        countStats.update(this);
    }

    public static ScrutariDB.Builder newBuilder(ScrutariDBName scrutariDBName, DataCoder dataCoder, AttributeKeyCoder attributeKeyCoder, DataCacheWriter dataCacheWriter, LexieCacheWriter lexieCacheWriter) {
        Builder builder = new Builder(scrutariDBName, dataCoder, attributeKeyCoder, dataCacheWriter, lexieCacheWriter);
        return builder;
    }


    public static class Builder {

        private ScrutariDB scrutariDB;
        private LexieAccessProviderBuilder lexieAccessProviderBuilder;
        private DataAccessProviderBuilder dataAccessProviderBuilder;
        private DataCoder dataCoder;

        private Builder(ScrutariDBName scrutariDBName, DataCoder dataCoder, AttributeKeyCoder attributeKeyCoder, DataCacheWriter dataCacheWriter, LexieCacheWriter lexieCacheWriter) {
            this.scrutariDB = new ScrutariDB(scrutariDBName);
            lexieAccessProviderBuilder = new LexieAccessProviderBuilder(attributeKeyCoder, lexieCacheWriter);
            dataAccessProviderBuilder = new DataAccessProviderBuilder(dataCacheWriter);
            this.dataCoder = dataCoder;
            scrutariDB.attributeKeyCoder = attributeKeyCoder;
        }

        public boolean containsBase(BaseURI baseURI) {
            return dataAccessProviderBuilder.containsBase(baseURI);
        }

        public ScrutariDB toScrutariDB() {
            scrutariDB.dataAccessProvider = dataAccessProviderBuilder.toDataAccessProvider();
            scrutariDB.lexieAccessProvider = lexieAccessProviderBuilder.toLexieAccessProvider();
            return scrutariDB;
        }

        public void addBase(BaseData.Builder baseDataBuilder) {
            int codebase = getCode(baseDataBuilder.getBaseURI());
            int corpusCount = baseDataBuilder.getCorpusDataBuilderCount();
            int[] codesCorpus = new int[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                CorpusData.Builder corpusDataBuilder = baseDataBuilder.getCorpusDataBuilder(i);
                int codecorpus = getCode(corpusDataBuilder.getCorpusURI());
                String hrefParent = corpusDataBuilder.getHrefParent();
                lexieAccessProviderBuilder.newCorpus(codecorpus);
                int ficheCount = corpusDataBuilder.getFicheDataBuilderCount();
                int[] codesFiches = new int[ficheCount];
                for (int j = 0; j < ficheCount; j++) {
                    FicheData.Builder ficheDataBuilder = corpusDataBuilder.getFicheDataBuilder(j);
                    int codefiche = getCode(ficheDataBuilder.getFicheURI());
                    FicheData ficheData = ficheDataBuilder.toFicheData(codefiche, hrefParent, codecorpus, codebase);
                    dataAccessProviderBuilder.addFicheData(ficheData);
                    lexieAccessProviderBuilder.initFicheData(ficheData);
                    codesFiches[j] = codefiche;
                }
                CorpusData corpusData = corpusDataBuilder.toCorpusData(codecorpus, codebase, codesFiches);
                dataAccessProviderBuilder.addCorpusData(corpusData);
                codesCorpus[i] = codecorpus;
            }
            int thesaurusCount = baseDataBuilder.getThesaurusDataBuilderCount();
            int[] codesThesaurus = new int[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                ThesaurusData.Builder thesaurusDataBuilder = baseDataBuilder.getThesaurusDataBuilder(i);
                int codethesaurus = getCode(thesaurusDataBuilder.getThesaurusURI());
                lexieAccessProviderBuilder.newThesaurus(codethesaurus);
                int motcleBuilderCount = thesaurusDataBuilder.getMotcleDataBuilderCount();
                int[] codesmotscles = new int[motcleBuilderCount];
                for (int j = 0; j < motcleBuilderCount; j++) {
                    MotcleData.Builder motcleDataBuilder = thesaurusDataBuilder.getMotcleDataBuilder(j);
                    int codemotcle = getCode(motcleDataBuilder.getMotcleURI());
                    MotcleData motcleData = motcleDataBuilder.toMotcleData(codemotcle, codethesaurus, codebase);
                    dataAccessProviderBuilder.addMotcleData(motcleData);
                    lexieAccessProviderBuilder.initMotcleData(motcleData);
                    codesmotscles[j] = codemotcle;
                }
                ThesaurusData thesaurusData = thesaurusDataBuilder.toThesaurusData(codethesaurus, codebase, codesmotscles);
                dataAccessProviderBuilder.addThesaurusData(thesaurusData);
                codesThesaurus[i] = codethesaurus;
            }
            BaseData baseData = baseDataBuilder.toBaseData(codebase, codesCorpus, codesThesaurus);
            dataAccessProviderBuilder.addBaseData(baseData);
        }

        private int getCode(ScrutariDataURI scrutariDataURI) {
            return dataCoder.getCode(scrutariDataURI);
        }

        private int checkCode(ScrutariDataURI scrutariDataURI) {
            return dataCoder.checkCode(scrutariDataURI);
        }

        public void setIndexation(IndexationBuilder indexationBuilder) {
            int groupCount = indexationBuilder.getGroupCount();
            for (int j = 0; j < groupCount; j++) {
                IndexationBuilder.Group group = indexationBuilder.getGroup(j);
                int codecorpus = checkCode(group.getCorpusURI());
                if (codecorpus == -1) {
                    continue;
                }
                int codethesaurus = checkCode(group.getThesaurusURI());
                if (codethesaurus == -1) {
                    continue;
                }
                int poidsMax = group.getPoidsMax();
                int indexationCount = group.getIndexationCount();
                for (int i = 0; i < indexationCount; i++) {
                    IndexationKey indexationKey = group.getIndexationKey(i);
                    int codefiche = checkCode(indexationKey.getFicheURI());
                    if (codefiche == -1) {
                        continue;
                    }
                    int codemotcle = checkCode(indexationKey.getMotcleURI());
                    if (codemotcle == -1) {
                        continue;
                    }
                    int poids = group.getPoids(i);
                    float ponderation = 0;
                    if (poids > 1) {
                        ponderation = ((float) poids) / ((float) (poidsMax + 1));
                    }
                    dataAccessProviderBuilder.addIndexation(codefiche, codecorpus, codemotcle, codethesaurus, ponderation);
                }
            }
        }

    }

}
