/* ScrutariLib_DB - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.db.codefilter;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.IndexationCodeList;
import net.mapeadores.util.conditions.ConditionConstants;


/**
 *
 * @author Vincent Calame
 */
public class CodeFilterUtils {

    public final static CodeFilter ALL_CODEFILTER = new AllCodeFilter();
    public final static CodeFilter NONE_CODEFILTER = new NoneCodeFilter();
    public final static CodeficheFilterResult NONE_CODEFICHEFILTERRESULT = new InternalCodeficheFilterResult(CodeListUtils.EMPTY_CODELIST, NONE_CODEFILTER);
    public final static MultiCodeCondition EMPTY_MULTICODECONDTION = new EmptyMultiCodeCondition();

    public static CodeFilter toCodeFilter(Set<Integer> set) {
        return new SetCodeFilter(set);
    }

    public static CodeficheFilterResult toIndexationCodeficheFilter(DataAccess dataAccess, MultiCodeCondition multiCodeCondition, CodeList codecorpusList) {
        int codeListCount = multiCodeCondition.getCodeListCount();
        if (codeListCount == 0) {
            return NONE_CODEFICHEFILTERRESULT;
        }
        if (codecorpusList == null) {
            codecorpusList = dataAccess.getCodecorpusList();
        }
        int corpusCount = codecorpusList.getCodeCount();
        if (corpusCount == 0) {
            return NONE_CODEFICHEFILTERRESULT;
        }
        short globalCondition = multiCodeCondition.getGlobalCondition();
        CodeficheFilterResultBuilder builder = null;
        if (globalCondition == ConditionConstants.MODE_AND) {
            builder = new AndCodeficheFilterResultBuilder(dataAccess, codecorpusList);
        } else {
            builder = new OrCodeficheFilterResultBuilder(dataAccess, codecorpusList);
        }
        for (int i = 0; i < codeListCount; i++) {
            builder.checkMotcleList(multiCodeCondition.getCodeList(i));
            if (builder.canStop()) {
                return builder.toCodeficheFilterResult();
            }
        }
        return builder.toCodeficheFilterResult();
    }


    private static class AllCodeFilter implements CodeFilter {

        public boolean acceptCode(int code) {
            return true;
        }

    }


    private static class NoneCodeFilter implements CodeFilter {

        public boolean acceptCode(int code) {
            return false;
        }

    }


    private static class SetCodeFilter implements CodeFilter {

        private Set<Integer> set;

        SetCodeFilter(Set<Integer> set) {
            this.set = set;
        }

        public boolean acceptCode(int code) {
            return set.contains(code);
        }

    }


    private static abstract class CodeficheFilterResultBuilder {

        DataAccess dataAccess;
        Set<Integer> resultCorpusSet = new LinkedHashSet<Integer>();
        Set<Integer> resultFicheSet;

        CodeficheFilterResultBuilder(DataAccess dataAccess, CodeList codecorpusList) {
            this.dataAccess = dataAccess;
            int corpusCount = codecorpusList.getCodeCount();
            for (int i = 0; i < corpusCount; i++) {
                resultCorpusSet.add(codecorpusList.getCode(i));
            }

        }

        abstract void checkMotcleList(CodeList codemotcleList);

        abstract boolean canStop();

        private CodeficheFilterResult toCodeficheFilterResult() {
            if (isEmpty()) {
                return NONE_CODEFICHEFILTERRESULT;
            }
            return new InternalCodeficheFilterResult(CodeListUtils.toCodeList(resultCorpusSet), CodeFilterUtils.toCodeFilter(resultFicheSet));
        }

        boolean isEmpty() {
            return (resultCorpusSet.isEmpty());
        }

    }


    private static class AndCodeficheFilterResultBuilder extends CodeficheFilterResultBuilder {

        private AndCodeficheFilterResultBuilder(DataAccess dataAccess, CodeList codecorpusList) {
            super(dataAccess, codecorpusList);

        }

        boolean canStop() {
            return isEmpty();
        }

        void checkMotcleList(CodeList codemotcleList) {
            int motcleCount = codemotcleList.getCodeCount();
            Set<Integer> previousFicheSet = resultFicheSet;
            Set<Integer> newFicheSet = new HashSet<Integer>();
            Set<Integer> newCorpusSet = new LinkedHashSet<Integer>();
            for (int codecorpus : resultCorpusSet) {
                for (int k = 0; k < motcleCount; k++) {
                    int codemotcle = codemotcleList.getCode(k);
                    IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                    int ficheCount = indexationCodeList.getCodeCount();
                    if (ficheCount > 0) {
                        boolean done = false;
                        for (int j = 0; j < ficheCount; j++) {
                            int codefiche = indexationCodeList.getCode(j);
                            if ((previousFicheSet == null) || (previousFicheSet.contains(codefiche))) {
                                done = true;
                                newFicheSet.add(codefiche);
                            }
                        }
                        if (done) {
                            newCorpusSet.add(codecorpus);
                        }
                    }
                }
            }
            resultCorpusSet = newCorpusSet;
            resultFicheSet = newFicheSet;
        }

    }


    private static class OrCodeficheFilterResultBuilder extends CodeficheFilterResultBuilder {

        private Set<Integer> startCorpusSet;

        private OrCodeficheFilterResultBuilder(DataAccess dataAccess, CodeList codecorpusList) {
            super(dataAccess, codecorpusList);
            resultFicheSet = new HashSet<Integer>();
            startCorpusSet = resultCorpusSet;
            resultCorpusSet = new LinkedHashSet<Integer>();
        }

        boolean canStop() {
            return false;
        }

        void checkMotcleList(CodeList codemotcleList) {
            int motcleCount = codemotcleList.getCodeCount();
            Set<Integer> newFicheSet = new HashSet<Integer>();
            for (int codecorpus : startCorpusSet) {
                int firstCodemotcle = codemotcleList.getCode(0);
                IndexationCodeList firstCodeList = dataAccess.getIndexationCodeList(firstCodemotcle, codecorpus);
                int firstFicheCount = firstCodeList.getCodeCount();
                if (firstFicheCount == 0) {
                    continue;
                }
                Set<Integer> ficheByCorpusSet = new HashSet<Integer>();
                for(int j = 0; j < firstFicheCount; j++) {
                    int codefiche = firstCodeList.getCode(j);
                    ficheByCorpusSet.add(codefiche);
                }
                for (int k = 1; k < motcleCount; k++) {
                    int codemotcle = codemotcleList.getCode(k);
                    IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                    int ficheCount = indexationCodeList.getCodeCount();
                    if (ficheCount > 0) {
                        for(int j = 0; j < ficheCount; j++) {
                            int codefiche = indexationCodeList.getCode(j);
                            if (ficheByCorpusSet.contains(codefiche)) {
                                newFicheSet.add(codefiche);
                            }
                        }
                        ficheByCorpusSet = newFicheSet;
                        if (ficheByCorpusSet.size() == 0) {
                            break;
                        }
                    } else {
                        ficheByCorpusSet.clear();
                        break;
                    }
                }
                if (!ficheByCorpusSet.isEmpty()) {
                    resultCorpusSet.add(codecorpus);
                    resultFicheSet.addAll(ficheByCorpusSet);
                }
            }
        }

    }


    private static class InternalCodeficheFilterResult implements CodeficheFilterResult {

        private CodeList codecorpusList;
        private CodeFilter codeficheFilter;

        private InternalCodeficheFilterResult(CodeList codecorpusList, CodeFilter codeficheFilter) {
            this.codecorpusList = codecorpusList;
            this.codeficheFilter = codeficheFilter;
        }

        public CodeList getCodecorpusList() {
            return codecorpusList;
        }

        public CodeFilter getCodeficheFilter() {
            return codeficheFilter;
        }

    }


    private static class EmptyMultiCodeCondition implements MultiCodeCondition {

        private EmptyMultiCodeCondition() {
        }

        public short getGlobalCondition() {
            return ConditionConstants.MODE_AND;
        }

        public int getCodeListCount() {
            return 0;
        }

        public CodeList getCodeList(int i) {
            return null;
        }

    }

}
