/* ScrutariLib_Data - Copyright (c) 2005-2011 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import net.scrutari.datauri.BaseURI;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.langinteger.LangInteger;
import java.util.List;
import java.util.ArrayList;
import java.io.Serializable;


/**
 *
 * @author Vincent Calame
 */
public final class BaseData implements Serializable {

    public final static short INTITULE_SHORT = 1;
    public final static short INTITULE_LONG = 2;
    private int codebase;
    private int[] codesCorpus;
    private int[] codesThesaurus;
    private LinkedIntHashMap intLangMap;
    private int[] langsUI;
    private String baseIconPart1;
    private String baseIconPart2;
    private BaseURI baseURI;

    public BaseData(BaseURI baseURI, int codebase, int[] codesCorpus, int[] codesThesaurus, LinkedIntHashMap intLangMap, int[] langsUI, String baseIcon) {
        this.codesCorpus = codesCorpus;
        this.codesThesaurus = codesThesaurus;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.langsUI = langsUI;
        this.baseURI = baseURI;
        parseBaseIcon(baseIcon);
    }

    public int getCodeBase() {
        return codebase;
    }

    public int getCorpusCount() {
        return codesCorpus.length;
    }

    public int getCodeCorpus(int i) {
        return codesCorpus[i];
    }

    public int getThesaurusCount() {
        return codesThesaurus.length;
    }

    public int getCodeThesaurus(int i) {
        return codesThesaurus[i];
    }

    public int getLangUiCount() {
        return langsUI.length;
    }

    public int getLangUi(int i) {
        return langsUI[i];
    }

    public BaseURI getBaseURI() {
        return baseURI;
    }

    public String getBaseIcon(int langUI) {
        if (baseIconPart2 == null) {
            return baseIconPart1;
        }
        StringBuffer buf = new StringBuffer(baseIconPart1.length() + baseIconPart2.length() + 10);
        buf.append(baseIconPart1);
        LangInteger.appendString(buf, langUI);
        buf.append(baseIconPart2);
        return buf.toString();
    }

    public int checkLangUi(int langInteger) {
        int length = langsUI.length;
        if (length == 0) {
            return langInteger;
        }
        for (int i = 0; i < length; i++) {
            if (langInteger == langsUI[i]) {
                return langInteger;
            }
        }
        return langsUI[0];
    }

    public BaseLangData getLangDataByLang(int langInt) {
        return (BaseLangData) intLangMap.get(langInt);
    }

    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée d'espaces blancs.
     */
    public String getLibelleStringByLang(short type, int langInteger) {
        BaseLangData baseLangInfo = getLangDataByLang(langInteger);
        if (baseLangInfo == null) {
            return null;
        }
        Libelle lib = baseLangInfo.getLibelle(type);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    public BaseLangData getLangDataByIndex(int i) {
        return (BaseLangData) intLangMap.getValueAt(i);
    }

    public int getLangDataCount() {
        return intLangMap.size();
    }


    public static class BaseLangData {

        private int langInteger;
        private Libelle shortLibelle;
        private Libelle longLibelle;

        private BaseLangData(int langInteger) {
            this.langInteger = langInteger;
        }

        public Libelle getLibelle(short type) {
            switch (type) {
                case INTITULE_SHORT:
                    return shortLibelle;
                case INTITULE_LONG:
                    return longLibelle;
            }
            return null;
        }

        public String getLibelleString(short type) {
            Libelle libelle = getLibelle(type);
            if (libelle == null) {
                return null;
            }
            return libelle.getLibelleString();
        }

        public int getLangInteger() {
            return langInteger;
        }

        private void setLibelle(short type, Libelle libelle) {
            switch (type) {
                case INTITULE_SHORT:
                    shortLibelle = libelle;
                    break;
                case INTITULE_LONG:
                    longLibelle = libelle;
                    break;
            }
        }

    }

    private void parseBaseIcon(String baseIcon) {
        if (baseIcon == null) {
            return;
        }
        int idx = baseIcon.indexOf("$LANGUI");
        if (idx == -1) {
            baseIconPart1 = baseIcon;
            baseIconPart2 = null;
        } else {
            baseIconPart1 = baseIcon.substring(0, idx);
            baseIconPart2 = baseIcon.substring(idx + 7);
        }
    }


    public static class Builder {

        private BaseURI baseURI;
        private List<CorpusData.Builder> corpusDataBuilderList = new ArrayList<CorpusData.Builder>();
        private List<ThesaurusData.Builder> thesaurusDataBuilderList = new ArrayList<ThesaurusData.Builder>();
        private List<Libelle> shortLibelleList = new ArrayList<Libelle>();
        private List<Libelle> longLibelleList = new ArrayList<Libelle>();
        private LinkedIntHashMap langsUiMap = new LinkedIntHashMap();
        private String baseIcon;

        public Builder(BaseURI baseURI) {
            this.baseURI = baseURI;
        }

        public void addLibelle(short type, Libelle libelle) {
            switch (type) {
                case BaseData.INTITULE_SHORT:
                    shortLibelleList.add(libelle);
                    break;
                case BaseData.INTITULE_LONG:
                    longLibelleList.add(libelle);
                    break;
            }
        }

        public void addLangUI(int langInteger) {
            langsUiMap.put(langInteger, Boolean.TRUE);
        }

        public BaseURI getBaseURI() {
            return baseURI;
        }

        public void setBaseIcon(String icon) {
            this.baseIcon = icon;
        }

        public void addCorpusDataBuilder(CorpusData.Builder builder) {
            corpusDataBuilderList.add(builder);
        }

        public void addThesaurusDataBuilder(ThesaurusData.Builder builder) {
            thesaurusDataBuilderList.add(builder);
        }

        public int getCorpusDataBuilderCount() {
            return corpusDataBuilderList.size();
        }

        public CorpusData.Builder getCorpusDataBuilder(int i) {
            return corpusDataBuilderList.get(i);
        }

        public int getThesaurusDataBuilderCount() {
            return thesaurusDataBuilderList.size();
        }

        public ThesaurusData.Builder getThesaurusDataBuilder(int i) {
            return thesaurusDataBuilderList.get(i);
        }

        public BaseData toBaseData(int codebase, int[] codesCorpus, int[] codesThesaurus) {
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            for (int i = 0; i < shortLibelleList.size(); i++) {
                Libelle libelle = shortLibelleList.get(i);
                putLibelle(linkedHashMap, libelle, BaseData.INTITULE_SHORT);
            }
            for (int i = 0; i < longLibelleList.size(); i++) {
                Libelle libelle = longLibelleList.get(i);
                putLibelle(linkedHashMap, libelle, BaseData.INTITULE_LONG);
            }
            int size = langsUiMap.size();
            int[] langsUI = new int[size];
            for (int i = 0; i < size; i++) {
                langsUI[i] = langsUiMap.getKeyAt(i);
            }
            return new BaseData(baseURI, codebase, codesCorpus, codesThesaurus, linkedHashMap, langsUI, baseIcon);
        }

        private void putLibelle(LinkedIntHashMap linkedHashMap, Libelle libelle, short type) {
            int langInteger = libelle.getLangInteger();
            BaseData.BaseLangData langInfo = (BaseData.BaseLangData) linkedHashMap.get(langInteger);
            if (langInfo == null) {
                langInfo = new BaseData.BaseLangData(langInteger);
                linkedHashMap.put(langInteger, langInfo);
            }
            langInfo.setLibelle(type, libelle);
        }

    }

}
