/* ScrutariLib_Data - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.data;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.CorpusURI;


/**
 *
 * @author Vincent Calame
 */
public final class CorpusData implements Serializable {

    public final static int INTITULECORPUS_FIELDNUMBER = -11;
    public final static int INTITULEFICHE_FIELDNUMBER = -22;
    private int codecorpus;
    private int codebase;
    private int[] codesFiches;
    private int complementCount;
    private LinkedIntHashMap intLangMap;
    private CorpusURI corpusURI;

    public CorpusData(CorpusURI corpusURI, int code, int codebase, int complementCount, int[] codesFiches, LinkedIntHashMap intLangMap) {
        this.codesFiches = codesFiches;
        this.codecorpus = code;
        this.codebase = codebase;
        this.intLangMap = intLangMap;
        this.complementCount = complementCount;
        this.corpusURI = corpusURI;
    }

    /**
     * Retourne le nombre maximum de champs complémentaires.
     */
    public int getComplementMaxNumber() {
        return complementCount;
    }

    public CorpusURI getCorpusURI() {
        return corpusURI;
    }

    /**
     * Retourne le code du corpus.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }

    /**
     * Retourne le code de la base.
     */
    public int getCodeBase() {
        return codebase;
    }

    /**
     * Retourne le total de fiches du corpus.
     */
    public int getFicheCount() {
        return codesFiches.length;
    }

    /**
     * Retourne le code de la i-ième fiche du corpus.
     */
    public int getCodeFiche(int i) {
        return codesFiches[i];
    }

    public CorpusLangData getLangDataByLang(int langInteger) {
        return (CorpusLangData) intLangMap.get(langInteger);
    }

    public int getFicheCountByLang(int langInteger) {
        CorpusLangData corpusLangInfo = getLangDataByLang(langInteger);
        if (corpusLangInfo == null) {
            return 0;
        }
        return corpusLangInfo.getFicheCount();
    }

    /**
     * Retourne null ou une chaîne de longueur non nulle et non composée
     * d'espaces blancs.
     */
    public String getLibelleStringByLang(int fieldNumber, int langInteger) {
        CorpusLangData corpusLangInfo = getLangDataByLang(langInteger);
        if (corpusLangInfo == null) {
            return null;
        }
        Libelle lib = corpusLangInfo.getLibelle(fieldNumber);
        if (lib == null) {
            return null;
        }
        return lib.getLibelleString();
    }

    public int getLangDataCount() {
        return intLangMap.size();
    }

    public CorpusLangData getLangDataByIndex(int index) {
        return (CorpusLangData) intLangMap.getValueAt(index);
    }


    public static class CorpusLangData {

        private int langInteger;
        private int ficheCount = 0;
        private Libelle corpusLibelle;
        private Libelle ficheLibelle;
        private Libelle[] complementsLibelle;

        CorpusLangData(int langInteger, int complementCount) {
            this.langInteger = langInteger;
            complementsLibelle = new Libelle[complementCount];
        }

        public int getFicheCount() {
            return ficheCount;
        }

        public Libelle getLibelle(int fieldNumber) {
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) {
                return corpusLibelle;
            }
            if (fieldNumber == INTITULEFICHE_FIELDNUMBER) {
                return ficheLibelle;
            }
            return complementsLibelle[fieldNumber - 1];
        }

        public String getLibelleString(int fieldNumber) {
            Libelle libelle = getLibelle(fieldNumber);
            if (libelle == null) {
                return null;
            }
            return libelle.getLibelleString();
        }

        public int getLangInteger() {
            return langInteger;
        }

        private void setLibelle(Libelle libelle, int fieldNumber) {
            if (fieldNumber == INTITULECORPUS_FIELDNUMBER) {
                corpusLibelle = libelle;
            } else if (fieldNumber == INTITULEFICHE_FIELDNUMBER) {
                ficheLibelle = libelle;
            } else {
                complementsLibelle[fieldNumber - 1] = libelle;
            }
        }

        private void increaseFicheCount() {
            ficheCount++;
        }

    }


    public static class Builder {

        private CorpusURI corpusURI;
        private List<Libelle> corpusLibelleList = new ArrayList<Libelle>();
        private List<Libelle> ficheLibelleList = new ArrayList<Libelle>();
        private List<List<Libelle>> libelleListOfList = new ArrayList<List<Libelle>>();
        private List<FicheData.Builder> ficheDataBuilderList = new ArrayList<FicheData.Builder>();
        private String hrefParent;

        public Builder(CorpusURI corpusURI) {
            this.corpusURI = corpusURI;
        }

        public CorpusURI getCorpusURI() {
            return corpusURI;
        }

        public void addLibelle(int fieldNumber, Libelle libelle) {
            if (libelle == null) {
                return;
            }
            if (fieldNumber == CorpusData.INTITULECORPUS_FIELDNUMBER) {
                corpusLibelleList.add(libelle);
            } else if (fieldNumber == CorpusData.INTITULEFICHE_FIELDNUMBER) {
                ficheLibelleList.add(libelle);
            } else if (fieldNumber > libelleListOfList.size()) {
                throw new IllegalArgumentException("fieldNumber > libelleListOfList.size()");
            } else {
                ((List<Libelle>) libelleListOfList.get(fieldNumber - 1)).add(libelle);
            }
        }

        public int createNewComplement() {
            int size = libelleListOfList.size();
            libelleListOfList.add(new ArrayList<Libelle>());
            return size;
        }

        public int getComplementMaxNumber() {
            return libelleListOfList.size();
        }

        public void addFicheDataBuilder(FicheData.Builder ficheDataBuilder) {
            if (ficheDataBuilder == null) {
                throw new IllegalArgumentException("ficheDataBuilder argument is null");
            }
            ficheDataBuilderList.add(ficheDataBuilder);
        }

        public int getFicheDataBuilderCount() {
            return ficheDataBuilderList.size();
        }

        public FicheData.Builder getFicheDataBuilder(int i) {
            return ficheDataBuilderList.get(i);
        }

        public CorpusData toCorpusData(int codecorpus, int codebase, int[] codesFiches) {
            LinkedIntHashMap linkedHashMap = new LinkedIntHashMap();
            int corpusLibelleCount = corpusLibelleList.size();
            for (int i = 0; i < corpusLibelleCount; i++) {
                Libelle libelle = corpusLibelleList.get(i);
                putLibelle(linkedHashMap, libelle, CorpusData.INTITULECORPUS_FIELDNUMBER);
            }
            int ficheLibelleCount = ficheLibelleList.size();
            for (int i = 0; i < ficheLibelleCount; i++) {
                Libelle libelle = ficheLibelleList.get(i);
                putLibelle(linkedHashMap, libelle, CorpusData.INTITULEFICHE_FIELDNUMBER);
            }
            int libelleListCount = libelleListOfList.size();
            for (int i = 0; i < libelleListCount; i++) {
                List<Libelle> liste = libelleListOfList.get(i);
                int compLibelleCount = liste.size();
                for (int j = 0; j < compLibelleCount; j++) {
                    Libelle libelle = liste.get(j);
                    putLibelle(linkedHashMap, libelle, i + 1);
                }
            }
            int size = ficheDataBuilderList.size();
            for (int i = 0; i < size; i++) {
                FicheData.Builder ficheDataBuilder = (FicheData.Builder) ficheDataBuilderList.get(i);
                int langInt = ficheDataBuilder.getLangInteger();
                CorpusData.CorpusLangData langInfo = (CorpusData.CorpusLangData) linkedHashMap.get(langInt);
                if (langInfo == null) {
                    langInfo = new CorpusData.CorpusLangData(langInt, libelleListOfList.size());
                    linkedHashMap.put(langInt, langInfo);
                }
                langInfo.increaseFicheCount();
            }
            return new CorpusData(corpusURI, codecorpus, codebase, libelleListOfList.size(), codesFiches, linkedHashMap);
        }

        private void putLibelle(LinkedIntHashMap linkedHashMap, Libelle libelle, int fieldNumber) {
            int langInt = libelle.getLangInteger();
            CorpusData.CorpusLangData langInfo = (CorpusData.CorpusLangData) linkedHashMap.get(langInt);
            if (langInfo == null) {
                langInfo = new CorpusData.CorpusLangData(langInt, libelleListOfList.size());
                linkedHashMap.put(langInt, langInfo);
            }
            langInfo.setLibelle(libelle, fieldNumber);
        }

        public String getHrefParent() {
            return hrefParent;
        }

        public void setHrefParent(String hrefParent) {
            this.hrefParent = StringUtils.nullifie(hrefParent);
        }

    }

}
