/* ScrutariLib_SearchEngine - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import java.util.SortedSet;
import java.util.TreeSet;
import net.mapeadores.scrutari.db.CodeFilter;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.IndexationCodeList;
import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.util.DataUtils;
import net.mapeadores.scrutari.searchengine.inner.SearchResultBuilder;
import net.mapeadores.scrutari.searchengine.pertinence.FichePertinence;
import net.mapeadores.scrutari.searchengine.pertinence.IndexationComparator;
import net.mapeadores.scrutari.searchengine.pertinence.IndexationPertinence;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.scrutari.searchengine.seeker.LexieSeeker;
import net.mapeadores.scrutari.searchengine.seeker.SeekOptions;
import net.mapeadores.scrutari.searchengine.seeker.SeekOptionsBuilder;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.SearchSequence;


/**
 *
 * @author Vincent Calame
 */
public class FicheSearchEngine {

    private ScrutariDB scrutariDB;
    private GlobalSearchOptions globalSearchOptions;
    private int searchTokenCount = 0;
    private int searchTokenMask = 0;
    private SearchResultBuilder searchResultBuilder;
    private SeekOptions seekOptions;
    private EngineSeekHandler engineSeekHandler;

    private FicheSearchEngine(SearchEngineParameters searchEngineParameters) {
        this.scrutariDB = searchEngineParameters.getScrutariDB();
        this.globalSearchOptions = searchEngineParameters.getGlobalSearchOptions();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SearchSequence searchSequence = searchEngineParameters.getSearchSequence();
        SearchOptions searchOptions = searchEngineParameters.getSearchOptions();
        SeekOptionsBuilder seekOptionsBuilder = new SeekOptionsBuilder(scrutariDB, dataAccess);
        seekOptionsBuilder.setSearchOptions(searchOptions);
        seekOptionsBuilder.setWithFiches(true);
        seekOptionsBuilder.setWithMotscles(true);
        this.seekOptions = seekOptionsBuilder.toSeekOptions();
        searchResultBuilder = new SearchResultBuilder(searchEngineParameters.getIdentifiant(), searchSequence, searchOptions);
        searchResultBuilder.setFicheMaximum(DataUtils.getFicheCountByCorpus(dataAccess, searchOptions.getCodecorpusList(), searchOptions.getLangIntegerArray()));
        this.searchTokenCount = searchSequence.getTokenCount();

        engineSeekHandler = new EngineSeekHandler(searchTokenCount);
        searchTokenMask = (int) Math.pow(2, searchTokenCount) - 1;
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        LexieSeeker lexieSeeker = new LexieSeeker(seekOptions, engineSeekHandler, lexieAccess);
        lexieSeeker.seek(searchSequence);
        lexieAccess.close();
        if (seekOptions.isWithMotscles()) {
            engineSeekHandler.checkMotclePertinence(searchOptions.getLangUi());
        }
        finalizeSearch(dataAccess);
        dataAccess.close();
    }

    /*
     * Finalise la recherche en conservant que les fiches qui remplissent tous
     * les critères et en les classant suivant la pertinence.
     */
    private void finalizeSearch(DataAccess dataAccess) {
        SortedSet<IndexationPertinence> indexationPertinenceSortedSet = searchIndexation(dataAccess);
        FicheSortEngine ficheSortEngine = new FicheSortEngine(searchTokenCount, searchTokenMask, globalSearchOptions);
        LinkedIntHashMap ficheCodeMap = engineSeekHandler.getFicheCodeMap();
        int mx = searchTokenMask;
        int ficheSize = ficheCodeMap.size();
        for (int i = 0; i < ficheSize; i++) {
            int codefiche = ficheCodeMap.getKeyAt(i);
            FichePertinence fichePertinence = (FichePertinence) ficheCodeMap.getValueAt(i);
            int bits = fichePertinence.getFicheCompletudeMask() | fichePertinence.getMotcleCompletudeMask();
            if ((bits & mx) == mx) {
                int dateInteger = dataAccess.getIntValue(DataAccess.FICHE_DATE_INTVALUE, codefiche);
                fichePertinence.setDateInteger(dateInteger);
                ficheSortEngine.addFichePertinence(fichePertinence);
            } else {
                fichePertinence.setPertinenceRate(0);
            }
        }
        ficheSortEngine.sort();
        searchResultBuilder.initFicheResult(ficheSortEngine);
        if (indexationPertinenceSortedSet != null) {
            for (IndexationPertinence indexationPertinence : indexationPertinenceSortedSet) {
                indexationPertinence.sortPertinenceFiche();
                searchResultBuilder.addMotcleResult(indexationPertinence);
            }
        }
    }

    /**
     * Modifie la pertinence des fiches en fonction de l'indexation par les
     * mots-clés sélectionné et en profite pour trier les mots-clés. Renvoie
     * l'ensemble trié.
     */
    private TreeSet<IndexationPertinence> searchIndexation(DataAccess dataAccess) {
        if (!seekOptions.isWithMotscles()) {
            return null;
        }
        LinkedIntHashMap ficheCodeMap = engineSeekHandler.getFicheCodeMap();
        LinkedIntHashMap motcleCodeMap = engineSeekHandler.getMotcleCodeMap();
        CodeList codecorpusList = seekOptions.getSearchFilter().getCodecorpusList();
        CodeFilter codeficheFilter = seekOptions.getSearchFilter().getCodeficheFilter();
        int corpusCount = codecorpusList.getCodeCount();
        int[] langIntArray = seekOptions.getLangIntegerArray();
        TreeSet<IndexationPertinence> indexationPertinenceSortedSet = new TreeSet<IndexationPertinence>(new IndexationComparator());
        int mx = searchTokenMask;
        int totalcomplets = 0;
        int motcleSize = motcleCodeMap.size();
        for (int i = 0; i < motcleSize; i++) {
            MotclePertinence motclePertinence = (MotclePertinence) motcleCodeMap.getValueAt(i);
            int codemotcle = motclePertinence.getCodeMotcle();
            IndexationPertinence indexationPertinence = new IndexationPertinence(motclePertinence);
            for (int k = 0; k < corpusCount; k++) {
                int codecorpus = codecorpusList.getCode(k);
                IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                int ficheCount = indexationCodeList.getCodeCount();
                for (int l = 0; l < ficheCount; l++) {
                    int codefiche = indexationCodeList.getCode(l);
                    if (!codeficheFilter.acceptCode(codefiche)) {
                        continue;
                    }
                    float ponderation = indexationCodeList.getPonderation(l);
                    FichePertinence fichePertinence = (FichePertinence) ficheCodeMap.get(codefiche);
                    if (fichePertinence == null) {
                        if (seekOptions.isAllLang()) {
                            fichePertinence = new FichePertinence(codefiche, codecorpus, searchTokenCount);
                            ficheCodeMap.put(codefiche, fichePertinence);
                        } else {
                            int langInteger = dataAccess.getIntValue(DataAccess.FICHE_LANG_INTVALUE, codefiche);
                            boolean contains = false;
                            int langLength = langIntArray.length;
                            for (int j = 0; j < langLength; j++) {
                                if (langIntArray[j] == langInteger) {
                                    contains = true;
                                    break;
                                }
                            }
                            if (contains) {
                                fichePertinence = new FichePertinence(codefiche, codecorpus, searchTokenCount);
                                ficheCodeMap.put(codefiche, fichePertinence);
                            } else {
                                continue;
                            }
                        }
                    }
                    fichePertinence.addIndexationPertinence(indexationPertinence, ponderation);
                }
            }
            if ((motclePertinence.getCompletudeMask() & mx) == mx) {
                totalcomplets++;
            }
            indexationPertinenceSortedSet.add(indexationPertinence);
        }
        searchResultBuilder.initMotcleCount(indexationPertinenceSortedSet.size());
        searchResultBuilder.setCompleteMotcleCount(totalcomplets);
        return indexationPertinenceSortedSet;
    }

    static public FicheSearchResult search(SearchEngineParameters searchEngineParameters) {
        FicheSearchEngine searchEngine = new FicheSearchEngine(searchEngineParameters);
        return searchEngine.searchResultBuilder.toFicheSearchResult();
    }

}
