/* ScrutariLib_SearchEngine - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine;

import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.pertinence.MotclePertinence;
import net.mapeadores.scrutari.searchengine.seeker.LexieSeeker;
import net.mapeadores.scrutari.searchengine.seeker.SeekOptions;
import net.mapeadores.scrutari.searchengine.seeker.SeekOptionsBuilder;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.text.SearchSequence;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.scrutari.db.LexieAccess;
import net.mapeadores.scrutari.searchengine.impl.MutableMotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class MotcleSearchEngine {

    private ScrutariDB scrutariDB;
    private SeekOptions seekOptions;
    private EngineSeekHandler engineSeekHandler;
    private int searchTokenCount = 0;
    private int searchTokenMask = 0;

    private MotcleSearchEngine(SearchEngineParameters searchEngineParameters) {
        this.scrutariDB = searchEngineParameters.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SearchSequence searchSequence = searchEngineParameters.getSearchSequence();
        SearchOptions searchOptions = searchEngineParameters.getSearchOptions();
        SeekOptionsBuilder seekOptionsBuilder = new SeekOptionsBuilder(scrutariDB, dataAccess);
        seekOptionsBuilder.setSearchOptions(searchOptions);
        seekOptionsBuilder.setWithFiches(false);
        seekOptionsBuilder.setWithMotscles(true);
        this.seekOptions = seekOptionsBuilder.toSeekOptions();
        this.searchTokenCount = searchSequence.getTokenCount();
        engineSeekHandler = new EngineSeekHandler(searchSequence.getTokenCount());
        searchTokenMask = (int) Math.pow(2, searchTokenCount) - 1;
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        LexieSeeker lexieSeeker = new LexieSeeker(seekOptions, engineSeekHandler, lexieAccess);
        lexieSeeker.seek(searchSequence);
        lexieAccess.close();
        dataAccess.close();
    }

    private MotcleSearchResult getResult() {
        LinkedIntHashMap motcleCodeMap = engineSeekHandler.getMotcleCodeMap();
        int size = motcleCodeMap.size();
        InternalMotcleSearchResult result = new InternalMotcleSearchResult();
        for (int i = 0; i < size; i++) {
            MotclePertinence motclePertinence = (MotclePertinence) motcleCodeMap.getValueAt(i);
            result.add(motclePertinence);
        }
        result.endInit();
        return result;
    }

    static public MotcleSearchResult search(SearchEngineParameters searchEngineParameters) {
        MotcleSearchEngine searchEngine = new MotcleSearchEngine(searchEngineParameters);
        return searchEngine.getResult();
    }


    private class InternalMotcleSearchResult implements MotcleSearchResult {

        private int allMotcleCount = 0;
        private int completeCount = 0;
        private MotcleSearchResultInfo[] completeArray;
        private MotcleSearchResultInfo[] otherArray;
        private SortedMap<MotclePertinence, MotcleSearchResultInfo> completeMap = new TreeMap<MotclePertinence, MotcleSearchResultInfo>();
        private SortedMap<MotclePertinence, MotcleSearchResultInfo> otherMap = new TreeMap<MotclePertinence, MotcleSearchResultInfo>();

        InternalMotcleSearchResult() {
        }

        public int getAllMotcleCount() {
            return allMotcleCount;
        }

        public int getCompleteMotcleCount() {
            return completeCount;
        }

        public MotcleSearchResultInfo getMotcleResult(int i) {
            if ((i < 0) || (i >= allMotcleCount)) {
                throw new IndexOutOfBoundsException();
            }
            if (i < completeCount) {
                return completeArray[i];
            } else {
                return otherArray[i - completeCount];
            }
        }

        void add(MotclePertinence motclePertinence) {
            MutableMotcleSearchResultInfo motcleSearchResultInfo = new MutableMotcleSearchResultInfo();
            motcleSearchResultInfo.setCode(motclePertinence.getCodeMotcle());
            motcleSearchResultInfo.setMotcleSearchTokenPositionListArray(SearchUtils.getMotcleSearchTokenPositionListArray(motclePertinence));
            if (motclePertinence.getCompletudeMask() == searchTokenMask) {
                completeMap.put(motclePertinence, motcleSearchResultInfo);
                completeCount++;
            } else {
                otherMap.put(motclePertinence, motcleSearchResultInfo);
            }
            allMotcleCount++;
        }

        void endInit() {
            if (completeCount > 0) {
                completeArray = new MotcleSearchResultInfo[completeCount];
                completeArray = completeMap.values().toArray(completeArray);
            }
            if (completeCount != allMotcleCount) {
                int otherCount = allMotcleCount - completeCount;
                otherArray = new MotcleSearchResultInfo[otherCount];
                otherArray = completeMap.values().toArray(otherArray);
            }
            completeMap = null;
            otherMap = null;
        }

    }

}
