/* ScrutariLib_SearchEngine - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.searchengine.pertinence;

import java.util.Comparator;
import net.mapeadores.scrutari.lexie.AlineaInteger;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.SubstringPosition;


/**
 * Classe servant au calcul de la pertinence d'une fiche. Les instances de cette
 * classe sont utilisées tout au long du traitement effectué par la classe
 * SearchEngine et servent à construire le résultat final via les instances de
 * SearchResult.
 *
 * @author Vincent Calame
 */
public class FichePertinence {

    static final AcceptedComparator ACCEPTED_COMPARATOR = new AcceptedComparator();
    private int codefiche;
    private int codecorpus;
    private int bestAlineaInteger = AlineaInteger.NOPERTINENT;
    private int[] alineaIntegerArray;
    private float[] ponderationArray;
    private float ponderationMax = 0;
    private int minimalBeginIndex = 9999;
    private SubstringPosition[] substringPositionArray;
    private MotclePertinence pertinencemotclemax;
    private MotclePertinence[] motclePertinenceArray;
    private int bitscompletudetexte = 0;
    private int bitscompletudemotscles = 0;
    private int searchTokenCountInTitre = 0;
    private int searchTokenCountInSoustitre = 0;
    private int searchTokenCountInMotscles = 0;
    private float pertinenceRate = 0;
    private int dateInteger;
    private int annee;

    /**
     * Construit une instance de FichePertinence pour une fiche donnée.
     */
    public FichePertinence(int codefiche, int codecorpus, int sequenceTokenCount) {
        this.codefiche = codefiche;
        this.codecorpus = codecorpus;
        alineaIntegerArray = new int[sequenceTokenCount];
        for (int i = 0; i < sequenceTokenCount; i++) {
            alineaIntegerArray[i] = AlineaInteger.NOPERTINENT;
        }
        substringPositionArray = new SubstringPosition[sequenceTokenCount];
        motclePertinenceArray = new MotclePertinence[sequenceTokenCount];
        ponderationArray = new float[sequenceTokenCount];
    }

    /**
     * Retourne le code de la fiche dont traite l'instance de FichePertinence.
     */
    public int getCodeFiche() {
        return codefiche;
    }

    /**
     * Retourne le code du corpus de la fiche dont traite l'instance de
     * FichePertinence.
     */
    public int getCodeCorpus() {
        return codecorpus;
    }

    /**
     * Retourne le « masque de complétude » de la fiche. Autrement dit,
     * l'information sur quel mot de la séquence de la recherche se trouve
     * effectivement dans la fiche (dans le titre, le sous-titre ou un champ
     * complémentaire). Un mot de la recherche se trouve dans la fiche si son «
     * bit » correspondant vaut 1.
     */
    public int getFicheCompletudeMask() {
        return bitscompletudetexte;
    }

    /**
     * Retourne le « masque de complétude » des mots-clés indexant la fiche.
     * Autrement dit, l'information sur quel mot de la séquence de la recherche
     * se trouve effectivement dans un des mots-clés qui indexe la fiche. Un mot
     * de la recherche se trouve dans un des mots-clés si son « bit »
     * correspondant vaut 1. <p>Si <em> getMotcleCompletudeMask()</em> retourne
     * 0, cela signifie qu'aucun mot-clé répondant à la recherche n'indexe la
     * fiche.
     */
    public int getMotcleCompletudeMask() {
        return bitscompletudemotscles;
    }

    /**
     * Retourne un tableau indiquant le numéro de l'alinéa dans lequel se trouve
     * chaque mot de la séquence de la recherche. Ce tableau a donc comme
     * longueur le nombre de mots de la séquence de recherche.
     */
    public int[] getAlineaIntegerByTokenArray() {
        return alineaIntegerArray;
    }

    /**
     * Retourne un tableau indiquant la position de chaque mot de la séquence de
     * recherche dans du champ dans lequel il se trouve chaque mot. Ce tableau a
     * donc comme longueur le nombre de mots de la séquence de recherche. <p>Si
     * le i-ème mot de la séquence ne se trouve dans aucun champ, la valeur du
     * tableau en i est nulle. On a donc une correspondance avec le tableau
     * renvoyé par la méthode <em>getFieldNumberByTokenArray()</em> puisque
     * (getFieldNumberByTokenArray()[i] == 0) ==
     * (getSubstringPositionByTokenArray()[i]);
     */
    public SubstringPosition[] getSubstringPositionByTokenArray() {
        return substringPositionArray;
    }

    /**
     * Retourne un tableau contenant les instances de MotclePertinence les plus
     * « pertinentes » pour chaque mot de la séquence de recherche. Ce tableau a
     * donc comme longueur le nombre de mots de la séquence de recherche. <p>si
     * le i-ème mot de la séquence ne se trouve dans aucun mot indexant la
     * fiche, la valeur du tableau en i est nulle. On a alors :
     * (getMotclePertinenceByTokenArray()[i] == null) ==
     * (getMotcleCompletudeMask() & Math.pow(i,2) == 0)
     */
    public MotclePertinence[] getMotclePertinenceByTokenArray() {
        return motclePertinenceArray;
    }

    public MotclePertinence getMotclePertinenceMax() {
        return pertinencemotclemax;
    }

    public float getPonderationMax() {
        return ponderationMax;
    }

    /**
     * Il s'agit du plus petit champ (le titre étant le champ le plus petit)
     * dans lequel se trouve un des mots de la recherche.
     */
    public int getBestAlineaInteger() {
        return bestAlineaInteger;
    }

    /**
     * Il s'agit du plus petit index auquel commence un des mots de la recherche
     * dans le plus petit champ. Cette valeur peut permettre de départager les
     * égalités de getMinimalFieldNumber()
     */
    public int getMinimalBeginIndex() {
        return minimalBeginIndex;
    }

    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans le
     * titre.
     */
    public int getSearchTokenCountInTitre() {
        return searchTokenCountInTitre;
    }

    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans le
     * sous-titre.
     */
    public int getSearchTokenCountInSoustitre() {
        return searchTokenCountInSoustitre;
    }

    /**
     * Il s'agit du nombre de mots de la recherche qui se trouvent dans les
     * mots-clés.
     */
    public int getSearchTokenCountInMotscles() {
        return searchTokenCountInMotscles;
    }

    /**
     * Taux de pertinence entre 0 et 1, c'est le taux qui a servi au classement
     * Retourne 0 si la fiche ne s'est pas révélée pertinente
     */
    public float getPertinenceRate() {
        return pertinenceRate;
    }

    /*
     * Ajoute une information de pertinence (numéro de l'alinéa, position dans
     * l'alinéa) pour le mot de la séquence de recherche à la position
     * tokenIndex. Cette méthode est abondamment utilisée dans la classe
     * SearchEngine.
     */
    public void addPertinence(int tokenIndex, int alineaInteger, SubstringPosition substringPosition) {
        bitscompletudetexte = (bitscompletudetexte | ((int) Math.pow(2, tokenIndex)));
        if (alineaInteger == AlineaInteger.NOPERTINENT) {
            return;
        }
        int currentAlineaInteger = alineaIntegerArray[tokenIndex];
        if (alineaInteger < currentAlineaInteger) {
            substringPositionArray[tokenIndex] = substringPosition;
            if (alineaInteger == AlineaInteger.TITRE) {
                searchTokenCountInTitre++;
                if (currentAlineaInteger == AlineaInteger.SOUSTITRE) {
                    searchTokenCountInSoustitre--;
                }
            } else if (alineaInteger == AlineaInteger.SOUSTITRE) {
                searchTokenCountInSoustitre++;
            }
            alineaIntegerArray[tokenIndex] = alineaInteger;

        } else if (alineaInteger == currentAlineaInteger) {
            if (substringPosition.getBeginIndex() < substringPositionArray[tokenIndex].getBeginIndex()) {
                substringPositionArray[tokenIndex] = substringPosition;
            }
        }
        if (alineaInteger < bestAlineaInteger) {
            bestAlineaInteger = alineaInteger;
            minimalBeginIndex = substringPosition.getBeginIndex();
        } else if (alineaInteger == bestAlineaInteger) {
            minimalBeginIndex = Math.min(minimalBeginIndex, substringPosition.getBeginIndex());
        }
    }

    /**
     * Ajoute une information de pertinence d'un mot-clé. Un seul mot-clé est
     * retenu par mot de la séquence comme mot-clé le plus pertinent. La valeur
     * relativeWeight donne le poids de l'indexation de la fiche par le mot-clé.
     * Cette valeur joue un rôle à la fois dans la sélection du mot-clé le plus
     * pertinent et dans le tri des instances de FichePertinence entre elles.
     * Cette méthode est abondamment utilisée dans la classe SearchEngine.
     */
    public void addIndexationPertinence(IndexationPertinence indexationPertinence, float ponderation) {
        indexationPertinence.addFichePertinence(this);
        MotclePertinence motclePertinence = indexationPertinence.getMotclePertinence();
        int bitsmotcle = motclePertinence.getCompletudeMask();
        for (int i = 0; i < motclePertinenceArray.length; i++) {
            int rg = (int) Math.pow(2, i);
            if ((bitsmotcle & rg) == rg) {
                if (motclePertinenceArray[i] == null) {
                    motclePertinenceArray[i] = motclePertinence;
                    ponderationArray[i] = ponderation;
                } else {
                    int compcompl = motclePertinence.compareCompletude(motclePertinenceArray[i]);
                    if (compcompl < 0) {
                        motclePertinenceArray[i] = motclePertinence;
                        ponderationArray[i] = ponderation;
                    } else if (compcompl == 0) {
                        if (ponderation > ponderationArray[i]) {
                            motclePertinenceArray[i] = motclePertinence;
                            ponderationArray[i] = ponderation;
                        } else if (ponderation == ponderationArray[i]) {
                            if (motclePertinence.compareTo(motclePertinenceArray[i]) < 0) {
                                motclePertinenceArray[i] = motclePertinence;
                            }
                        }
                    }
                }
                if ((bitscompletudemotscles & rg) == 0) {
                    bitscompletudemotscles = bitscompletudemotscles | rg;
                    searchTokenCountInMotscles++;
                }
            }
        }
        if (pertinencemotclemax == null) {
            pertinencemotclemax = motclePertinence;
            ponderationMax = ponderation;
        } else {
            int compcompl = motclePertinence.compareCompletude(pertinencemotclemax);
            if (compcompl < 0) {
                pertinencemotclemax = motclePertinence;
                ponderationMax = ponderation;
            } else if (compcompl == 0) {
                if (ponderation > ponderationMax) {
                    pertinencemotclemax = motclePertinence;
                    ponderationMax = ponderation;
                } else if (ponderation == ponderationMax) {
                    if (motclePertinence.compareTo(pertinencemotclemax) < 0) {
                        pertinencemotclemax = motclePertinence;
                    }
                }
            }
        }
    }

    /**
     * Taux de pertinence final compris entre 0 et 1
     */
    public void setPertinenceRate(float pertinenceRate) {
        this.pertinenceRate = pertinenceRate;
    }

    /*
     * Comparateur qui intervient en fin de traitement, une fois que les valeurs
     * de pertinence ont été fixées et qui sert à trier les fiches en fonction
     * de cette valeur. Ce comparateur n'est intéressant que pour les fiches
     * déjà triées.
     */

    static class AcceptedComparator implements Comparator<FichePertinence> {

        public int compare(FichePertinence pf1, FichePertinence pf2) {
            if (pf1 == pf2) {
                return 0;
            }
            if (pf1.pertinenceRate < pf2.pertinenceRate) {
                return -1;
            } else {
                return 1;
            }
        }

    }

    public int getDateInteger() {
        return dateInteger;
    }

    public void setDateInteger(int dateInteger) {
        this.dateInteger = dateInteger;
        if (dateInteger != DateInteger.NULL) {
            annee = DateInteger.getAnnee(dateInteger);
        } else {
            annee = DateInteger.NULL;
        }
    }

    public int getAnnee() {
        return annee;
    }

}
