/* ScrutariLib_Xml - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.scrutari.xml.xmlpull;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.ScrutariAttributes;
import net.mapeadores.scrutari.xml.ErrorKeys;
import net.mapeadores.scrutari.xml.Parameters;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Libelle;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LibelleElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;


/**
 *
 * @author Vincent Calame
 */
public class CorpusHandler extends DefaultElementHandler implements ErrorKeys {

    private static final int HREF_PARENT = 1;
    private static final int TITRE = 3;
    private static final int SOUSTITRE = 4;
    private static final int DATE = 5;
    private static final int LANG = 6;
    private static final int HREF = 7;
    private static final int COMPLEMENT = 8;
    private static final int FICHE_ICON = 9;
    private static final int LATITUDE = 10;
    private static final int LONGITUDE = 11;
    private static final int VALUE = 12;
    private Parameters parameters;
    private boolean metadataDone = false;
    private CorpusURI corpusURI;
    private CorpusData.Builder corpusDataBuilder;
    private FicheHandler ficheHandler = new FicheHandler();

    public CorpusHandler(Map<String, String> attributeMap, Parameters parameters) {
        String corpusName = getAttributeString(attributeMap, "corpus-name");
        if (corpusName == null) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_MISSINGCORPUSNAME);
            throw new IllegalArgumentException();
        }
        this.parameters = parameters;
        corpusURI = new CorpusURI(parameters.getBaseURI(), corpusName);
        if (!parameters.testDoublon(corpusURI)) {
            parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_DOUBLECORPUSNAME, corpusURI);
            throw new IllegalArgumentException();
        }
        corpusURI = (CorpusURI) parameters.intern(corpusURI);
        corpusDataBuilder = new CorpusData.Builder(corpusURI);
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("corpus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_MISSINGMETADATA, corpusURI);
                return null;
            }
        } else {
            if (tagname.equals("fiche")) {
                String ficheId = getAttributeString(attributeMap, "fiche-id");
                if (ficheId == null) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGFICHEID);
                    return null;
                }
                FicheURI ficheURI = new FicheURI(corpusURI, ficheId);
                if (!parameters.testDoublon(ficheURI)) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_DOUBLEFICHEID, ficheURI);
                    return null;
                }
                ficheHandler.setFicheURI(ficheURI);
                return ficheHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof FicheHandler) {
            FicheData.Builder ficheDataBuilder = ((FicheHandler) elementHandler).getFicheDataBuilder();
            corpusDataBuilder.addFicheDataBuilder(ficheDataBuilder);
        }
    }

    public CorpusData.Builder getCorpusDataBuilder() {
        if (!metadataDone) {
            return null;
        }
        return corpusDataBuilder;
    }


    private class MetadataHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("intitule-corpus")) {
                return new IntituleHandler(CorpusData.INTITULECORPUS_FIELDNUMBER);
            } else if (tagname.equals("intitule-fiche")) {
                return new IntituleHandler(CorpusData.INTITULEFICHE_FIELDNUMBER);
            } else if (tagname.equals("complement-metadata")) {
                int index = corpusDataBuilder.createNewComplement();
                return new IntituleHandler(index + 1);
            } else if (tagname.equals("href-parent")) {
                return new SimpleElementHandler(HREF_PARENT);
            } else if (tagname.equals("corpus-icon")) {
                return null;//return new SimpleElementHandler(TYPE);
            } else {
                return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case HREF_PARENT:
                    corpusDataBuilder.setHrefParent(text);
                    break;
            }
        }

    }


    private class FicheHandler extends DefaultElementHandler {

        FicheURI ficheURI;
        private FicheData.Builder ficheDataBuilder;

        FicheHandler() {
        }

        void setFicheURI(FicheURI ficheURI) {
            this.ficheURI = ficheURI;
            ficheDataBuilder = new FicheData.Builder(ficheURI, corpusDataBuilder.getComplementMaxNumber());
        }

        FicheData.Builder getFicheDataBuilder() {
            return ficheDataBuilder;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("titre")) {
                return new SimpleElementHandler(TITRE);
            } else if (tagname.equals("soustitre")) {
                return new SimpleElementHandler(SOUSTITRE);
            } else if (tagname.equals("date")) {
                return new SimpleElementHandler(DATE);
            } else if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            } else if (tagname.equals("href")) {
                return new SimpleElementHandler(HREF);
            } else if (tagname.equals("complement")) {
                return new SimpleElementHandler(COMPLEMENT);
            } else if (tagname.equals("fiche-icon")) {
                return new SimpleElementHandler(FICHE_ICON);
            } else if (tagname.equals("geoloc")) {
                return new GeolocHandler();
            } else if (tagname.equals("attr")) {
                String localKey = attributeMap.get("key");
                if (localKey != null) {
                    localKey = localKey.trim();
                }
                if ((localKey == null) || (localKey.isEmpty())) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGLOCALKEY, ficheURI);
                    return null;
                }
                String nameSpace = attributeMap.get("ns");
                if (nameSpace != null) {
                    nameSpace = nameSpace.trim();
                }
                if ((nameSpace == null) || (nameSpace.isEmpty())) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGNAMESPACE, ficheURI);
                    return null;
                }
                try {
                    AttributeKey attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
                    return new AttrHandler(attributeKey);
                } catch (ParseException pe) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_WRONGATTRIBUTEKEY, ficheURI);
                    return null;
                }
            } else {
                return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case TITRE:
                    ficheDataBuilder.setTitre(text);
                    break;
                case SOUSTITRE:
                    ficheDataBuilder.setSoustitre(text);
                    break;
                case DATE:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setDateInteger(DateInteger.parse(text));
                        }
                    } catch (DateIntegerException ide) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_DATEERROR, ficheURI);
                    }
                    break;
                case LANG:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setLangInteger(LangInteger.parse(text));
                        }
                    } catch (LangIntegerException ile) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_LANGERROR, ficheURI);
                    }
                    break;
                case HREF:
                    ficheDataBuilder.setHref(text);
                    break;
                case COMPLEMENT:
                    boolean done = ficheDataBuilder.addComplement(text);
                    if (!done) {
                        parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_TOMANYCOMPLEMENTS, ficheURI);
                    }
                    break;
                case FICHE_ICON:
                    ficheDataBuilder.setFicheIcon(text);
                    break;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof GeolocHandler) {
                GeolocHandler geolocHandler = (GeolocHandler) elementHandler;
                String latitude = geolocHandler.latitude;
                String longitude = geolocHandler.longitude;
                boolean done = true;
                if (latitude == null) {
                    done = false;
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGLATITUDE);
                }
                if (longitude == null) {
                    done = false;
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_FICHE_MISSINGLONGITUDE);
                }
                if (done) {
                    ficheDataBuilder.setGeoloc(latitude, longitude);
                }
            } else if (elementHandler instanceof AttrHandler) {
                AttrHandler attrHandler = (AttrHandler) elementHandler;
                ficheDataBuilder.addAttribute(attrHandler.attributeKey, attrHandler.valueList);
            }
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        int fieldNumber;

        IntituleHandler(int fieldNumber) {
            this.fieldNumber = fieldNumber;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LibelleElementHandler(attributeMap);
                } catch (LangIntegerException lie) {
                    parameters.getScrutariXmlErrorHandler().addWarningLog(WARNING_XML_CORPUS_LANGERROR, corpusURI);
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Libelle libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                corpusDataBuilder.addLibelle(fieldNumber, libelle);
            }
        }

    }


    private class GeolocHandler extends DefaultElementHandler {

        private String latitude;
        private String longitude;

        GeolocHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lat")) {
                return new SimpleElementHandler(LATITUDE);
            } else if (tagname.equals("lon")) {
                return new SimpleElementHandler(LONGITUDE);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case LATITUDE:
                    latitude = text;
                    break;
                case LONGITUDE:
                    longitude = text;
                    break;
            }
        }

    }


    private class AttrHandler extends DefaultElementHandler {

        private AttributeKey attributeKey;
        private List<CleanedString> valueList = new ArrayList<CleanedString>();

        private AttrHandler(AttributeKey attributeKey) {
            this.attributeKey = attributeKey;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("val")) {
                return new SimpleElementHandler(VALUE);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case VALUE:
                    CleanedString cs = CleanedString.newInstance(text);
                    if (cs != null) {
                        valueList.add(cs);
                    }
                    break;
            }
        }

    }

}
