/* SctServer - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver;

import fr.exemole.sctserver.admin.OptionsManager;
import fr.exemole.sctserver.admin.ScrutariInfo;
import fr.exemole.sctserver.admin.ScrutariInfoUtils;
import fr.exemole.sctserver.admin.SourceInfoManager;
import fr.exemole.sctserver.admin.UrlCollector;
import fr.exemole.sctserver.conf.ConfConstants;
import fr.exemole.sctserver.conf.ConfDirectory;
import fr.exemole.sctserver.conf.ConfFileInfo;
import fr.exemole.sctserver.conf.ConfIOException;
import fr.exemole.sctserver.feed.SctFeed;
import fr.exemole.sctserver.feed.SctFeedBuilder;
import fr.exemole.sctserver.request.RequestInfo;
import fr.exemole.sctserver.searchlog.MutableQueryLog;
import fr.exemole.sctserver.storage.LogAccess;
import fr.exemole.sctserver.storage.SctServerStorage;
import fr.exemole.sctserver.storage.SctStorageException;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.db.AttributeKeyCoder;
import net.mapeadores.scrutari.db.AttributeKeyCoderBuilder;
import net.mapeadores.scrutari.db.DataCoder;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.ScrutariDBName;
import net.mapeadores.scrutari.db.ScrutariDataURICodeList;
import net.mapeadores.scrutari.searchengine.FicheSearchEngine;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.MotcleSearchEngine;
import net.mapeadores.scrutari.searchengine.MotcleSearchResult;
import net.mapeadores.scrutari.searchengine.PertinencePonderation;
import net.mapeadores.scrutari.searchengine.SearchEngineParameters;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchEngine;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchResult;
import net.mapeadores.scrutari.supermotcle.SupermotcleManager;
import net.mapeadores.scrutari.xml.ScrutariXmlErrorHandler;
import net.mapeadores.scrutari.xml.XmlPullReader;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.logging.LogConstants;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MutableProcessLog;
import net.mapeadores.util.logging.ProcessLog;
import net.mapeadores.util.logging.ProcessLogItem;
import net.mapeadores.util.logging.ProcessLogUtils;
import net.mapeadores.util.logging.ProcessMessage;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.logging.TimeLog;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.text.SearchSequence;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.tree.URITree;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class SctServer implements LogConstants, ConfConstants {

    public final static String MALFORMED_XML_CATEGORY = "MALFORMED_XML";
    public final static String SEVERE_CATEGORY = "SEVERE";
    public final static String WARNING_CATEGORY = "WARNING";
    private final static short INIT_TYPE = 1;
    private final static short FORCE_TYPE = 2;
    private final static short CRON_TYPE = 3;
    private final static short UPDATECONF_TYPE = 4;
    private final static DateFormat dateformattotal = new SimpleDateFormat("yyyyMMdd");
    private String name;
    private SctServerStorage sctServerStorage;
    private SourceInfoManager sourceInfoManager;
    private ScrutariDB scrutariDB;
    private final SupermotcleManager supermotcleManager = new SupermotcleManager();
    private int idrequete = 1;
    private int lastDateInteger = DateInteger.current();
    private Timer timer;
    private OptionsManager optionsManager;
    private boolean hasUpdateProcessLog = false;
    private SearchLog searchLog;
    private InternalConfState confState = new InternalConfState();
    private boolean onReload = false;
    private SctFeed sctFeed;

    public SctServer(String name, SctServerStorage sctServerStorage) {
        this.name = name;
        this.sctServerStorage = sctServerStorage;
        this.searchLog = sctServerStorage.getSearchLogAccess().getSearchLog();
        this.sourceInfoManager = new SourceInfoManager();
        this.optionsManager = new OptionsManager(this);
        reload(INIT_TYPE);
        this.timer = new Timer(true);
        timer.scheduleAtFixedRate(new UpdateTask(), getFirstTime(), getPeriod());
    }

    public String getName() {
        return name;
    }

    public boolean hasUpdateProcessLog() {
        return hasUpdateProcessLog;
    }

    public boolean reload(boolean forceDataDownload) {
        short type = (forceDataDownload) ? FORCE_TYPE : INIT_TYPE;
        boolean done = reload(type);
        if (done) {
            sctServerStorage.cleanSearchResults();
        }
        return done;
    }

    private boolean reload(short type) {
        if (onReload) {
            return false;
        }
        onReload = true;
        Reloader reloader = new Reloader(type);
        reloader.reload();
        onReload = false;
        return true;
    }

    public ScrutariDB getScrutariDB() {
        return scrutariDB;
    }

    public SourceInfoManager getScrutariInfoManager() {
        return sourceInfoManager;
    }

    public OptionsManager getOptionsManager() {
        return optionsManager;
    }

    public SupermotcleManager getSupermotcleManager() {
        synchronized (supermotcleManager) {
            if (!supermotcleManager.isInit()) {
                supermotcleManager.update(scrutariDB);
            }
        }
        return supermotcleManager;
    }

    public FicheSearchResult getSearchResult(String identifiant) {
        return sctServerStorage.getFicheSearchResult(identifiant);
    }

    public LogAccess getLogAccess() {
        return sctServerStorage.getLogAccess();
    }

    public SearchLog getSearchLog() {
        return searchLog;
    }

    public SctServerStorage getSctServerStorage() {
        return sctServerStorage;
    }

    public FicheSearchResult newSearch(SearchSequence searchSequence, SearchOptions searchOptions, RequestInfo requestInfo) {
        String identifiant = getNewIdentifiant(searchSequence, searchOptions, requestInfo);
        SearchEngineParameters searchEngineParameters = new SearchEngineParameters();
        searchEngineParameters.setScrutariDB(scrutariDB);
        searchEngineParameters.setIdentifiant(identifiant);
        searchEngineParameters.setSearchOptions(searchOptions);
        searchEngineParameters.setSearchSequence(searchSequence);
        searchEngineParameters.setGlobalSearchOptions(optionsManager.getGlobalSearchOptions());
        FicheSearchResult ficheSearchResult = FicheSearchEngine.search(searchEngineParameters);
        sctServerStorage.saveFicheSearchResult(ficheSearchResult);
        logSearch(ficheSearchResult, requestInfo);
        return ficheSearchResult;
    }

    public MotcleSearchResult newMotcleSearch(SearchSequence searchSequence, SearchOptions searchOptions) {
        SearchEngineParameters searchEngineParameters = new SearchEngineParameters();
        searchEngineParameters.setScrutariDB(scrutariDB);
        searchEngineParameters.setSearchOptions(searchOptions);
        searchEngineParameters.setSearchSequence(searchSequence);
        MotcleSearchResult motcleSearchResult = MotcleSearchEngine.search(searchEngineParameters);
        return motcleSearchResult;
    }

    public SupermotcleSearchResult newSupermotcleSearch(SearchSequence searchSequence, SearchOptions searchOptions) {
        SearchEngineParameters searchEngineParameters = new SearchEngineParameters();
        searchEngineParameters.setSearchOptions(searchOptions);
        searchEngineParameters.setSearchSequence(searchSequence);
        searchEngineParameters.setSupermotcleManager(getSupermotcleManager());
        SupermotcleSearchResult supermotcleSearchResult = SupermotcleSearchEngine.search(searchEngineParameters);
        return supermotcleSearchResult;
    }

    private synchronized void logSearch(FicheSearchResult searchResult, RequestInfo requestInfo) {
        if (!optionsManager.filterLog(requestInfo)) {
            return;
        }
        MutableQueryLog queryLog = new MutableQueryLog();
        queryLog.setSearchSequence(searchResult.getSearchSequence().toString());
        queryLog.setSite(requestInfo.getSite());
        queryLog.setFicheCount(searchResult.getFicheCount());
        queryLog.setAllMotcleCount(searchResult.getMotcleCount());
        queryLog.setCompleteMotcleCount(searchResult.getCompleteMotcleCount());
        queryLog.setFicheMaximum(searchResult.getFicheMaximum());
        sctServerStorage.getSearchLogAccess().log(this, queryLog);
    }

    public boolean updateConf() {
        boolean done = reload(UPDATECONF_TYPE);
        return done;
    }

    public ConfState getConfState() {
        return confState;
    }

    public SctFeed getFeed() {
        return sctFeed;
    }

    private boolean checkSourceInfo(UpdateProcessLog updateProcessLog, boolean forceDataDownload) {
        boolean dataChange = false;
        UrlCollector urlCollector = new UrlCollector(sctServerStorage, updateProcessLog);
        int count = getScrutariInfoManager().getSourceCount();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = getScrutariInfoManager().getSourceInfo(i);
            urlCollector.setScrutariInfo(scrutariInfo);
            urlCollector.collectData(forceDataDownload);
            short newState = urlCollector.getFinalState();
            int update = urlCollector.getLastUpdate();
            if (ScrutariInfoUtils.hasChange(scrutariInfo, newState, update)) {
                dataChange = true;
            }
            sourceInfoManager.setState(scrutariInfo.getKeyString(), newState);
            sourceInfoManager.setScrutariDataUrl(scrutariInfo.getKeyString(), urlCollector.getDataURL());
            sourceInfoManager.setLastUpdate(scrutariInfo.getKeyString(), update);
        }
        if (forceDataDownload) {
            return true;
        } else {
            return dataChange;
        }
    }

    private void saveUpdateProcessLog(UpdateProcessLog updateProcessLog) {
        hasUpdateProcessLog = saveProcessLog(updateProcessLog, "updateprocess");
    }

    private boolean saveProcessLog(ProcessLog processLog, String name) {
        boolean hasLog = processLog.hasLog();
        try {
            OutputStream outputStream = sctServerStorage.getLogAccess().getLogOutputStream("log_" + name + ".txt");
            Writer writer = new OutputStreamWriter(outputStream, "UTF-8");
            writer.write(ProcessLogUtils.toString(processLog));
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return hasLog;
    }

    private boolean saveProcessLog(ProcessURI processURI, ProcessMessageList processMessageList, String name) {
        try {
            OutputStream outputStream = sctServerStorage.getLogAccess().getLogOutputStream("log_" + name + ".txt");
            Writer writer = new OutputStreamWriter(outputStream, "UTF-8");
            LogUtils.append(writer, processURI, processMessageList);
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return (processMessageList.getProcessMessageCount() > 0);
    }

    public String getLog(String name) {
        try {
            return IOUtils.toString(sctServerStorage.getLogAccess().getLogDocStream("log_" + name + ".txt").getInputStream());
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    private void reinitScrutariDB(UpdateProcessLog updateProcessLog, ScrutariDataURICodeList initialScrutariDataURICodeList, Map<String, AttributeDefList> attributeDefListMap) {
        boolean logStarted = false;
        try {
            if (!TimeLog.isStarted()) {
                logStarted = true;
                TimeLog.startLog(sctServerStorage.getLogAccess().getLogOutputStream("log_time.txt"), "SctServer.reinitScrutariDB()", "Début de la réinitialisation");
            } else {
                TimeLog.log("SctServer.reinitScrutariDB()", "Début de la réinitialisation");
            }
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        ScrutariDB newScrutariDB = newScrutariDB(updateProcessLog, initialScrutariDataURICodeList, attributeDefListMap);
        ScrutariDB oldScrutariDB = scrutariDB;
        scrutariDB = newScrutariDB;
        supermotcleManager.clear();
        reinitSctFeed();
        if (oldScrutariDB != null) {
            TimeLog.log("SctServer.reinitScrutariDB()", "Début de la suppression de la base existante");
            ScrutariDBName scrutariDbName = oldScrutariDB.getScrutariDBName();
            sctServerStorage.clearCache(scrutariDbName);
            TimeLog.log("SctServer.reinitScrutariDB()", "Fin de la suppression de la base existante");
        }
        if (logStarted) {
            try {
                TimeLog.endLog("SctServer.reinitScrutariDB()", "Fin de la réinitialisation");
            } catch (IOException ioe) {
                throw new NestedIOException(ioe);
            }
        } else {
            TimeLog.log("SctServer.reinitScrutariDB()", "Fin de la réinitialisation");
        }
    }

    private void reinitSctFeed() {
        SctFeedBuilder builder = new SctFeedBuilder();
        builder.build(this);
        sctFeed = builder.toSctFeed();
    }

    private ScrutariDB newScrutariDB(UpdateProcessLog updateProcessLog, ScrutariDataURICodeList initialScrutariDataURICodeList, Map<String, AttributeDefList> attributeDefListMap) {
        TimeLog.log("SctServer.newScrutariDB()", "Début de la création de la nouvelle base");
        ScrutariDBName scrutariDbName = ScrutariDBName.newName();
        URITree initialUriTree = sctServerStorage.getURITree(null);
        DataCoder dataCoder = DataCoder.newInstance(initialScrutariDataURICodeList, initialUriTree);
        AttributeKeyCoder attributeKeyCoder = AttributeKeyCoderBuilder.build(attributeDefListMap);
        ScrutariDB.Builder builder = ScrutariDB.newBuilder(scrutariDbName, dataCoder, attributeKeyCoder, sctServerStorage.getDataCacheWriter(scrutariDbName), sctServerStorage.getLexieCacheWriter(scrutariDbName));
        TimeLog.log("SctServer.newScrutariDB()", "Fin de la création / début du remplissage");
        builWithXmlPull(builder, updateProcessLog);
        TimeLog.log("SctServer.newScrutariDB()", "Début de la conversion finale");
        ScrutariDB newScrutariDB = builder.toScrutariDB();
        TimeLog.log("SctServer.newScrutariDB()", "Fin de la conversion / début des stats");
        newScrutariDB.updateStats();
        TimeLog.log("SctServer.newScrutariDB()", "Fin des stats / base finie");
        sctServerStorage.saveURITree(dataCoder.toURITree(), null);
        if (dataCoder.hasNewURI()) {
            sctServerStorage.saveURITree(dataCoder.getNewURITree(), scrutariDbName);
        }
        return newScrutariDB;
    }

    private void builWithXmlPull(ScrutariDB.Builder builder, UpdateProcessLog updateProcessLog) {
        XmlPullReader xmlPullReader = new XmlPullReader(builder, updateProcessLog);
        int count = sourceInfoManager.getSourceCount();
        for (int i = 0; i < count; i++) {
            ScrutariInfo scrutariInfo = sourceInfoManager.getSourceInfo(i);
            URL url = scrutariInfo.getUrl();
            String urlString = url.toString();
            updateProcessLog.setCurrentURL(url);
            TimeLog.log(urlString, "Début du traitement des données");
            parseXmlPull(xmlPullReader, scrutariInfo, updateProcessLog);
            TimeLog.log(urlString, "Fin du traitement des données");
        }
        xmlPullReader.flush();
    }

    private void parseXmlPull(XmlPullReader xmlPullReader, ScrutariInfo scrutariInfo, UpdateProcessLog updateProcessLog) {
        InputStream inputStream = sctServerStorage.getScrutariData(scrutariInfo);
        TimeLog.log("SctServer.parseXmlPull()", "Récupération des données faites / début de la lecture");
        if (inputStream == null) {
            return;
        }
        XmlPullParser xpp = null;
        try {
            XmlPullParserFactory factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
            xpp = factory.newPullParser();
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
        try {
            try {
                xpp.setInput(inputStream, null);
                BaseURI baseURI = xmlPullReader.process(xpp);
                sourceInfoManager.setBaseURI(scrutariInfo.getKeyString(), baseURI);

            } catch (XmlPullParserException xppe) {
                sourceInfoManager.setState(scrutariInfo.getKeyString(), ScrutariInfo.DATA_NOTXML_STATE);
                updateProcessLog.xmlPullParserException(xppe);
            } catch (EOFException ioe) {
                sourceInfoManager.setState(scrutariInfo.getKeyString(), ScrutariInfo.DATA_NOTXML_STATE);
                updateProcessLog.xmlPullParserException(ioe);
            } finally {
                inputStream.close();
            }

        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }

    }

    private Document getScrutariDataDocument(ScrutariInfo scrutariInfo, UpdateProcessLog updateProcessLog) {
        if (scrutariInfo.getState() != ScrutariInfo.OK_STATE) {
            return null;
        }
        InputStream inputStream = sctServerStorage.getScrutariData(scrutariInfo);
        if (inputStream == null) {
            return null;
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(inputStream);
            inputStream.close();
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        } catch (SAXException saxe) {
            sourceInfoManager.setState(scrutariInfo.getKeyString(), ScrutariInfo.DATA_NOTXML_STATE);
            updateProcessLog.saxException(saxe);
            return null;
        }
        return doc;
    }

    private Date getFirstTime() {
        Calendar calendar = Calendar.getInstance();
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(Calendar.DAY_OF_MONTH, day + 1);
        calendar.set(Calendar.HOUR_OF_DAY, (int) (4 * Math.random()));
        calendar.set(Calendar.MINUTE, (int) (60 * Math.random()));
        return calendar.getTime();
    }

    private long getPeriod() {
        return 24 * 60 * 60 * 1000;
    }

    private synchronized String getNewIdentifiant(SearchSequence sequence, SearchOptions options, RequestInfo requestInfo) {
        Date current = new Date();
        int currentDate = DateInteger.fromDate(current);
        if (currentDate != lastDateInteger) {
            idrequete = 1;
            lastDateInteger = currentDate;
        }
        String identifiant = dateformattotal.format(current) + "-" + String.valueOf(idrequete);
        idrequete++;
        return identifiant;
    }


    private class Reloader {

        private short type;
        private ConfDirectory confDirectory;

        Reloader(short type) {
            this.type = type;
        }

        public void reload() {
            initConfDirectory();
            sourceInfoManager.reloadInfoUrlArray(confDirectory.getInfoUrlKeyArray());
            optionsManager.reloadSearchLogDenyArray(confDirectory.getSearchLogDenyArray());
            if (type != UPDATECONF_TYPE) {
                UpdateProcessLog updateProcessLog = new UpdateProcessLog();
                boolean forceReload = (type == FORCE_TYPE);
                boolean done = checkSourceInfo(updateProcessLog, forceReload);
                if ((type != CRON_TYPE) || (done)) {
                    reinitScrutariDB(updateProcessLog, confDirectory.getInitialScrutariDataURICodeList(), confDirectory.getAttributeDefListMap());
                    saveUpdateProcessLog(updateProcessLog);
                    sctServerStorage.cleanSearchResults();
                }
            }
            optionsManager.reloadCorpusClasseArray(confDirectory.getCorpusClasseArray());
            PertinencePonderation globalPertinencePonderation = confDirectory.getGlobalPertinencePonderation();
            if (globalPertinencePonderation != null) {
                optionsManager.setGlobalPertinencePonderation(globalPertinencePonderation.getFraicheur(), globalPertinencePonderation.getRarete(), globalPertinencePonderation.getPosition());
            } else {
                optionsManager.setGlobalPertinencePonderation(0.25f, 0.15f, 0.6f);
            }
        }

        private void initConfDirectory() {
            confDirectory = sctServerStorage.getConfDirectory();
            try {
                confDirectory.reload();
                checkProcessLog(SOURCES_NAME);
                checkProcessLog(ATTRIBUTES_NAME);
                checkProcessLog(CORPUS_CLASSES_NAME);
                checkProcessLog(SEARCHLOG_FILTER_NAME);
                checkProcessLog(URI_CODES_NAME);
                checkProcessLog(FEED_NAME);
                confState.setOkState();
            } catch (ConfIOException cioe) {
                confState.setGlobalConfigException(cioe);
            }
        }

        private void checkProcessLog(String name) {
            ConfFileInfo confFileInfo = confDirectory.getConfFileInfo(name);
            if (!confFileInfo.isHere()) {
                confState.setFileState(name, FILE_MISSING);
            } else {
                ProcessMessageList processMessageList = confFileInfo.getProcessMessageList();
                int count = processMessageList.getProcessMessageCount();
                if (count > 0) {
                    int errorType = 0;
                    for (int i = 0; i < count; i++) {
                        ProcessMessage processMessage = processMessageList.getProcessMessage(i);
                        String category = processMessage.getCategory();
                        if (category.equals(MALFORMED_XML_CATEGORY)) {
                            errorType = 2;
                            break;
                        } else if (category.equals(SEVERE_CATEGORY)) {
                            errorType = 1;
                        }
                    }
                    if (errorType == 2) {
                        confState.setFileState(name, FILE_XML_MALFORMED);
                    } else if (errorType == 1) {
                        confState.setFileState(name, FILE_XML_ERROR);
                    } else {
                        confState.setFileState(name, FILE_XML_WARNING);
                    }
                } else {
                    confState.setFileState(name, FILE_OK);
                }
                saveProcessLog(confFileInfo.getProcessURI(), processMessageList, name);
            }

        }

    }


    private class UpdateTask extends TimerTask {

        public void run() {
            reload(CRON_TYPE);
        }

    }


    private class UpdateProcessLog extends MutableProcessLog implements ScrutariXmlErrorHandler, UrlCollector.CollectErrorHandler {

        public void setCurrentURL(URL url) {
            setCurrentURI(ProcessURI.fromURL(url));
        }

        public void saxException(SAXException saxe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(saxe.getMessage());
        }

        public void xmlPullParserException(Exception xppe) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(MALFORMED_XML_CATEGORY);
            logItem.setKey(SctServerLog.SEVERE_DATA_BASEFILE_XML);
            logItem.setObject(xppe.getMessage());
        }

        public void addSevereLog(String key, ScrutariDataURI mapeadoresURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(SEVERE_CATEGORY);
            logItem.setKey(key);
            logItem.setObject(mapeadoresURI);
        }

        public void addWarningLog(String key, ScrutariDataURI mapeadoresURI) {
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(WARNING_CATEGORY);
            logItem.setKey(key);
            logItem.setObject(mapeadoresURI);
        }

        public void addSevereLog(String key) {
            addSevereLog(key, null);
        }

        public void addWarningLog(String key) {
            addWarningLog(key, null);
        }

        public void warningDownload(URL url, String message) {
            setCurrentURL(url);
            ProcessLogItem logItem = newLogItemInstance();
            logItem.setCategory(WARNING_CATEGORY);
            logItem.setKey("warning.download");
            logItem.setObject(message);
        }

    }


    private class InternalConfState implements ConfState {

        private short state = GLOBAL_UNDEFINED;
        private String confIOMessage = null;
        private Map<String, Short> stateMap = new HashMap<String, Short>();

        private void setGlobalConfigException(ConfIOException ioe) {
            state = GLOBAL_CONFIO_ERROR;
            confIOMessage = ioe.getFullMessage();
        }

        private void clear() {
            this.confIOMessage = null;
            this.state = GLOBAL_UNDEFINED;
            stateMap.clear();
        }

        public String getConfIOMessage() {
            return confIOMessage;
        }

        public short getGlobalState() {
            return state;
        }

        void setOkState() {
            this.state = GLOBAL_OK;
        }

        void setFileState(String name, short fileState) {
            stateMap.put(name, new Short(fileState));
        }

        public short getFileState(String name) {
            Short sh = stateMap.get(name);
            if (sh == null) {
                return FILE_STATE_UNKNOWN;
            }
            return sh.shortValue();
        }

    }

}
