/* SctServer - Copyright (c) 2009-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.conf;

import fr.exemole.sctserver.admin.SearchLogDeny;
import fr.exemole.sctserver.feed.SctFeedMetadata;
import java.io.File;
import java.io.IOException;
import java.util.Map;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.scrutari.db.ScrutariDataURICodeList;
import net.mapeadores.scrutari.searchengine.CorpusClasse;
import net.mapeadores.scrutari.searchengine.PertinencePonderation;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.logging.ProcessMessageList;
import net.mapeadores.util.logging.ProcessURI;
import net.mapeadores.util.text.UrlKey;


/**
 *
 * @author Vincent Calame
 */
public class DefaultConfDirectory implements ConfDirectory {

    private File confDir;
    private InternalConfFileInfo sourcesFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo corpusClassesFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo searchLogFilterFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo uriCodesFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo feedFileInfo = new InternalConfFileInfo();
    private InternalConfFileInfo attributesFileInfo = new InternalConfFileInfo();
    private UrlKey[] sourceUrlKeyArray = null;
    private CorpusClasse[] corpusClasseArray = null;
    private SearchLogDeny[] searchLogDenyArray = null;
    private PertinencePonderation globalPertinencePonderation;
    private ScrutariDataURICodeList initialScrutariDataURICodeList;
    private SctFeedMetadata sctFeedMetadata;
    private Map<String, AttributeDefList> attributeDefListMap;

    public DefaultConfDirectory(File confDir) {
        this.confDir = confDir;
    }

    public UrlKey[] getInfoUrlKeyArray() {
        return sourceUrlKeyArray;
    }

    public CorpusClasse[] getCorpusClasseArray() {
        return corpusClasseArray;
    }

    public PertinencePonderation getGlobalPertinencePonderation() {
        return globalPertinencePonderation;
    }

    public SearchLogDeny[] getSearchLogDenyArray() {
        return searchLogDenyArray;
    }

    public ScrutariDataURICodeList getInitialScrutariDataURICodeList() {
        return initialScrutariDataURICodeList;
    }

    public SctFeedMetadata getSctFeedMetadata() {
        return sctFeedMetadata;
    }
    
    public Map<String, AttributeDefList> getAttributeDefListMap() {
        return attributeDefListMap;
    }

    public void reload() throws ConfIOException {
        clear();
        try {
            if (!confDir.exists()) {
                boolean done = confDir.mkdirs();
                if (!done) {
                    throw new ConfIOException("confDirectory cannot be created : " + confDir.getPath());
                }
            } else if (!confDir.isDirectory()) {
                throw new ConfIOException("confDirectory is not a directory : " + confDir.getPath());
            }
            File[] fileList = confDir.listFiles();
            for (int i = 0; i < fileList.length; i++) {
                File f = fileList[i];
                if (!f.isDirectory()) {
                    String name = f.getName();
                    if ((name.length() > 4) && (name.endsWith(".xml"))) {
                        String baseName = name.substring(0, name.length() - 4);
                        if (baseName.equals(SOURCES_NAME)) {
                            initSources(f);
                        } else if (baseName.equals(CORPUS_CLASSES_NAME)) {
                            initCorpusClasses(f);
                        } else if (baseName.equals(SEARCHLOG_FILTER_NAME)) {
                            initSearchLogFilter(f);
                        } else if (baseName.equals(URI_CODES_NAME)) {
                            initUriCodes(f);
                        } else if (baseName.equals(FEED_NAME)) {
                            initFeed(f);
                        } else if (baseName.equals(ATTRIBUTES_NAME)) {
                            initAttributes(f);
                        }
                    }
                }
            }
        } catch (SecurityException se) {
            throw new ConfIOException(se);
        } catch (IOException ioe) {
            throw new ConfIOException(ioe);
        } catch (ParserConfigurationException pe) {
            throw new ConfIOException(pe);
        }
    }

    private void clear() {
        sourcesFileInfo.clear();
        corpusClassesFileInfo.clear();
        searchLogFilterFileInfo.clear();
        uriCodesFileInfo.clear();
        feedFileInfo.clear();
        sourceUrlKeyArray = null;
        corpusClasseArray = null;
        searchLogDenyArray = null;
        initialScrutariDataURICodeList = null;
        sctFeedMetadata = null;
        attributeDefListMap = null;
    }

    private void initSources(File f) throws IOException, ParserConfigurationException {
        sourcesFileInfo.setHere(true);
        SourcesConf sourcesConf = SourcesConf.load(f);
        sourcesFileInfo.setProcessMessageList(sourcesConf.getProcessMessageList());
        sourcesFileInfo.setProcessURI(sourcesConf.getProcessURI());
        sourceUrlKeyArray = sourcesConf.getInfoUrlKeyArray();
    }

    private void initCorpusClasses(File f) throws IOException, ParserConfigurationException {
        corpusClassesFileInfo.setHere(true);
        CorpusClassesConf corpusClassesConf = CorpusClassesConf.load(f);
        corpusClassesFileInfo.setProcessMessageList(corpusClassesConf.getProcessMessageList());
        corpusClassesFileInfo.setProcessURI(corpusClassesConf.getProcessURI());
        corpusClasseArray = corpusClassesConf.getCorpusClasseArray();
        globalPertinencePonderation = corpusClassesConf.getGlobalPertinencePonderation();
    }

    private void initSearchLogFilter(File f) throws IOException, ParserConfigurationException {
        searchLogFilterFileInfo.setHere(true);
        SearchLogFilterConf searchLogFilterConf = SearchLogFilterConf.load(f);
        searchLogFilterFileInfo.setProcessMessageList(searchLogFilterConf.getProcessMessageList());
        searchLogFilterFileInfo.setProcessURI(searchLogFilterConf.getProcessURI());
        searchLogDenyArray = searchLogFilterConf.getSearchLogDenyArray();
    }

    private void initUriCodes(File f) throws IOException, ParserConfigurationException {
        uriCodesFileInfo.setHere(true);
        UriCodesConf uriCodesConf = UriCodesConf.load(f);
        uriCodesFileInfo.setProcessMessageList(uriCodesConf.getProcessMessageList());
        uriCodesFileInfo.setProcessURI(uriCodesConf.getProcessURI());
        initialScrutariDataURICodeList = uriCodesConf.getScrutariDataURICodeList();
    }

    private void initFeed(File f) throws IOException, ParserConfigurationException {
        feedFileInfo.setHere(true);
        FeedConf feedConf = FeedConf.load(f);
        feedFileInfo.setProcessMessageList(feedConf.getProcessMessageList());
        feedFileInfo.setProcessURI(feedConf.getProcessURI());
        sctFeedMetadata = feedConf.getSctFeedMetadata();
    }
    
    private void initAttributes(File f) throws IOException, ParserConfigurationException {
        attributesFileInfo.setHere(true);
        AttributesConf attributesConf = AttributesConf.load(f);
        attributesFileInfo.setProcessMessageList(attributesConf.getProcessMessageList());
        attributesFileInfo.setProcessURI(attributesConf.getProcessURI());
        attributeDefListMap = attributesConf.getAttributeDefListMap();
    }

    public ConfFileInfo getConfFileInfo(String name) {
        if (name.equals(SOURCES_NAME)) {
            return sourcesFileInfo;
        } else if (name.equals(CORPUS_CLASSES_NAME)) {
            return corpusClassesFileInfo;
        } else if (name.equals(SEARCHLOG_FILTER_NAME)) {
            return searchLogFilterFileInfo;
        } else if (name.equals(URI_CODES_NAME)) {
            return uriCodesFileInfo;
        } else if (name.equals(FEED_NAME)) {
            return feedFileInfo;
        } else if (name.equals(ATTRIBUTES_NAME)) {
            return attributesFileInfo;
        } else {
            throw new IllegalArgumentException("Unknown name : " + name);
        }
    }


    private static class InternalConfFileInfo implements ConfFileInfo {

        private boolean here = false;
        private ProcessURI processURI;
        private ProcessMessageList processMessageList;

        private InternalConfFileInfo() {
        }

        private void clear() {
            this.here = false;
            this.processURI = null;
            this.processMessageList = null;
        }

        public boolean isHere() {
            return here;
        }

        public ProcessURI getProcessURI() {
            return processURI;
        }

        public ProcessMessageList getProcessMessageList() {
            return processMessageList;
        }

        private void setHere(boolean here) {
            this.here = here;
        }

        private void setProcessMessageList(ProcessMessageList processMessageList) {
            this.processMessageList = processMessageList;
        }

        private void setProcessURI(ProcessURI processURI) {
            this.processURI = processURI;
        }

    }

}
