/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.feed.xml;

import fr.exemole.sctserver.SctContext;
import fr.exemole.sctserver.SctContextConstants;
import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.feed.FeedOptions;
import fr.exemole.sctserver.feed.SctFeed;
import fr.exemole.sctserver.feed.SctFeedEntry;
import fr.exemole.sctserver.feed.SctFeedMetadata;
import java.io.IOException;
import java.util.Date;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDBName;
import net.mapeadores.util.text.LibelleHolder;
import net.mapeadores.util.xml.AppendableXMLPart;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.ns.NameSpace;
import net.scrutari.datauri.tree.URITree;


/**
 *
 * @author Vincent Calame
 */
public class AtomFeedXMLPart extends AppendableXMLPart {

    private SctServer sctServer;
    private SctContext sctContext;
    private SctFeed sctFeed;
    private String feedId;
    private FeedOptions feedOptions;

    public AtomFeedXMLPart(AppendableXMLWriter xmlWriter, SctContext sctContext, FeedOptions feedOptions) {
        super(xmlWriter);
        this.sctContext = sctContext;
        this.sctServer = sctContext.getSctServer();
        this.feedOptions = feedOptions;
        this.feedId = initFeedId(sctContext);
        sctFeed = sctServer.getFeed();
    }

    public void appendFeed() throws IOException {
        DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
        startOpenTag("feed");
        XMLUtils.appendNameSpaceAttribute(this, NameSpace.ATOM_NAMESPACE);
        endOpenTag();
        addSimpleElement("title", getTitle());
        addSimpleElement("updated", sctFeed.getFeedUpdated());
        openTag("author");
        addSimpleElement("name", getAuthorName());
        closeTag("author");
        addSimpleElement("id", feedId);
        int entryCount = sctFeed.getSctFeedEntryCount();
        for (int i = 0; i < entryCount; i++) {
            SctFeedEntry entry = sctFeed.getSctFeedEntry(i);
            appendFeedEntry(entry, dataAccess);
        }
        closeTag("feed");
        dataAccess.close();
    }

    private String getTitle() {
        String title = null;
        SctFeedMetadata metadata = sctFeed.getMetadata();
        if (metadata != null) {
            LibelleHolder titleLib = metadata.getAuthorName();
            if (titleLib != null) {
                title = titleLib.getLibelleString(feedOptions.getWorkingLangInteger(), false);
            }
        }
        if ((title == null) || (title.length() == 0)) {
            title = "Scrutari " + sctServer.getName();
        }
        return title;
    }

    private String getAuthorName() {
        String authorName = null;
        SctFeedMetadata metadata = sctFeed.getMetadata();
        if (metadata != null) {
            LibelleHolder authorNameLib = metadata.getAuthorName();
            if (authorNameLib != null) {
                authorName = authorNameLib.getLibelleString(feedOptions.getWorkingLangInteger(), false);
            }
        }
        if ((authorName == null) || (authorName.length() == 0)) {
            authorName = sctServer.getName();
        }
        return authorName;
    }

    private void appendFeedEntry(SctFeedEntry entry, DataAccess dataAccess) throws IOException {
        Date date = entry.getDate();
        URITree uriTree = sctServer.getSctServerStorage().getURITree(date);
        if (uriTree == null) {
            return;
        }
        EntryContent entryContent = EntryContentFactory.newEntryContent(dataAccess, uriTree, feedOptions, sctServer.getOptionsManager().getGlobalSearchOptions());
        int ficheAddCount = entryContent.getFicheAddCount();
        int motcleAddCount = entryContent.getMotcleAddCount();
        if ((ficheAddCount == 0) && (motcleAddCount == 0)) {
            return;
        }
        openTag("entry");
        addSimpleElement("title", getEntryTitle(date, ficheAddCount, motcleAddCount));
        addSimpleElement("updated", ScrutariDBName.toGmtIsoFormat(date));
        addSimpleElement("id", feedId + "/" + ScrutariDBName.toNameFormat(date));
        startOpenTag("content");
        addAttribute("type", "xhtml");
        endOpenTag();
        startOpenTag("div");
        XMLUtils.appendNameSpaceAttribute(this, NameSpace.XHTML_NAMESPACE);
        endOpenTag();
        entryContent.addContent(this);
        closeTag("div");
        closeTag("content");
        closeTag("entry");
    }

    private String getEntryTitle(Date date, int ficheAddCount, int motcleAddCount) {
        StringBuilder buf = new StringBuilder();
        if (feedOptions.isTechnicalTitle()) {
            buf.append("s=");
            buf.append(sctServer.getName());
            buf.append(";d=");
            buf.append(ScrutariDBName.toGmtIsoFormat(date));
            buf.append(";f=");
            buf.append(ficheAddCount);
            if (!feedOptions.isFichesOnly()) {
                buf.append(";m=");
                buf.append(motcleAddCount);
            }
            return buf.toString();
        } else {
            buf.append(sctServer.getName());
            buf.append(" + ");
            buf.append(ficheAddCount);
            if (motcleAddCount > 0) {
                buf.append(" / +");
                buf.append(motcleAddCount);
            }
        }
        return buf.toString();
    }

    private String initFeedId(SctContext sctContext) {
        StringBuilder buf = new StringBuilder();
        buf.append("scrutarifeed:");
        buf.append("/");
        buf.append((String) sctContext.getContextObject(SctContextConstants.UUID_NAME));
        buf.append("/");
        buf.append(sctServer.getName());
        return buf.toString();
    }

}
