/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.feed.xml;

import fr.exemole.sctserver.feed.FeedOptions;
import java.io.IOException;
import java.util.Comparator;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.util.BaseCheck;
import net.mapeadores.scrutari.searchengine.CorpusClasse;
import net.mapeadores.scrutari.searchengine.GlobalSearchOptions;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.URITree;


/**
 *
 * @author Vincent Calame
 */
public class ClassesEntryContent implements EntryContent {

    private FeedOptions feedOptions;
    private int ficheAddCount = 0;
    private SortedMap<Integer, FicheGroup> ficheGroupByClasseMap;
    private FicheGroup uniqueFicheGroup;
    private GlobalSearchOptions globalSearchOptions;

    public ClassesEntryContent(FeedOptions feedOptions, GlobalSearchOptions globalSearchOptions) {
        this.feedOptions = feedOptions;
        this.globalSearchOptions = globalSearchOptions;
        int corpusClasseCount = globalSearchOptions.getCorpusClasseCount();
        if (corpusClasseCount == 0) {
            uniqueFicheGroup = new FicheGroup(null);
        } else {
            ficheGroupByClasseMap = new TreeMap<Integer, FicheGroup>(new InverseComparator());
            for (int i = 0; i < corpusClasseCount; i++) {
                CorpusClasse corpusClasse = globalSearchOptions.getCorpusClasse(i);
                ficheGroupByClasseMap.put(corpusClasse.getRank(), new FicheGroup(corpusClasse));
            }
        }
    }

    public int getFicheAddCount() {
        return ficheAddCount;
    }

    public int getMotcleAddCount() {
        return 0;
    }

    public void addContent(XMLWriter xmlWriter) throws IOException {
        if (uniqueFicheGroup != null) {
            uniqueFicheGroup.add(xmlWriter, feedOptions);
        } else {
            for (FicheGroup ficheGroup : ficheGroupByClasseMap.values()) {
                ficheGroup.add(xmlWriter, feedOptions);
            }
        }
    }

    void checkUriTree(DataAccess dataAccess, URITree uriTree) {
        int workingLangInteger = feedOptions.getWorkingLangInteger();
        int ct = 0;
        int baseCount = uriTree.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseNode baseNode = uriTree.getBaseNode(i);
            BaseURI baseURI = baseNode.getBaseURI();
            BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
            if (baseData == null) {
                continue;
            }
            BaseCheck baseCheck = BaseCheck.check(baseData, workingLangInteger);
            int corpusCount = baseNode.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                CorpusNode corpusNode = baseNode.getCorpusNode(j);
                CorpusURI corpusURI = corpusNode.getCorpusURI();
                CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
                if (corpusData == null) {
                    continue;
                }
                FicheGroup ficheGroup = getFicheGroup(corpusData.getCodeCorpus());
                int ficheCount = corpusNode.getFicheCount();
                for (int k = 0; k < ficheCount; k++) {
                    FicheNode ficheNode = corpusNode.getFicheNode(k);
                    FicheURI ficheURI = ficheNode.getFicheURI();
                    FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                    if (ficheData == null) {
                        continue;
                    }
                    ficheGroup.addFicheData(ficheData, baseCheck);
                    ct++;
                }
            }
        }
        this.ficheAddCount = ct;
    }

    private FicheGroup getFicheGroup(int codecorpus) {
        if (uniqueFicheGroup != null) {
            return uniqueFicheGroup;
        }
        int rank = globalSearchOptions.getCorpusClasseRank(codecorpus);
        return ficheGroupByClasseMap.get(rank);
    }


    private static class FicheGroup {

        private CorpusClasse corpusClasse;
        private SortedSet<FicheDataWrapper> ficheDataWrapperSet;

        private FicheGroup(CorpusClasse corpusClasse) {
            this.corpusClasse = corpusClasse;
            ficheDataWrapperSet = new TreeSet<FicheDataWrapper>(new FicheDataWrapperComparator());
        }

        private void addFicheData(FicheData ficheData, BaseCheck baseCheck) {
            ficheDataWrapperSet.add(new FicheDataWrapper(ficheData, baseCheck));
        }

        public void add(XMLWriter xmlWriter, FeedOptions feedOptions) throws IOException {
            if (ficheDataWrapperSet.isEmpty()) {
                return;
            }
            if (corpusClasse != null) {
                xmlWriter.startOpenTag("p", true);
                xmlWriter.addAttribute("class", "SCT_FicheGroup");
                xmlWriter.endOpenTag(false);
                xmlWriter.addText(corpusClasse.getLibelleString(feedOptions.getWorkingLangInteger(), true));
                xmlWriter.closeTag("p", false, false);
            }
            xmlWriter.startOpenTag("ul");
            xmlWriter.addAttribute("class", "SCT_UL_Fiche");
            xmlWriter.endOpenTag();
            for (FicheDataWrapper ficheDataWrapper : ficheDataWrapperSet) {
                ficheDataWrapper.add(xmlWriter, feedOptions);
            }
            xmlWriter.closeTag("ul");
        }

    }


    private static class FicheDataWrapper {

        private FicheData ficheData;
        private BaseCheck baseCheck;

        private FicheDataWrapper(FicheData ficheData, BaseCheck baseCheck) {
            this.ficheData = ficheData;
            this.baseCheck = baseCheck;

        }

        private FicheData getFicheData() {
            return ficheData;
        }

        private void add(XMLWriter xmlWriter, FeedOptions feedOptions) throws IOException {
            EntryContentUtils.addFicheData(xmlWriter, ficheData, feedOptions, baseCheck);
        }

    }


    private static class FicheDataWrapperComparator implements Comparator<FicheDataWrapper> {

        public int compare(FicheDataWrapper ficheDataWrapper1, FicheDataWrapper ficheDataWrapper2) {
            FicheData ficheData1 = ficheDataWrapper1.getFicheData();
            FicheData ficheData2 = ficheDataWrapper2.getFicheData();
            String titre1 = ficheData1.getTitre();
            String titre2 = ficheData2.getTitre();
            if (titre1 == null) {
                if (titre2 != null) {
                    return 1;
                }
                return compareCode(ficheData1, ficheData2);
            } else {
                if (titre2 == null) {
                    return -1;
                }
                int comp = titre1.compareToIgnoreCase(titre2);
                if (comp != 0) {
                    return comp;
                } else {
                    return compareCode(ficheData1, ficheData2);
                }
            }
        }

        private int compareCode(FicheData ficheData1, FicheData ficheData2) {
            int code1 = ficheData1.getCodeFiche();
            int code2 = ficheData2.getCodeFiche();
            if (code1 < code2) {
                return -1;
            }
            if (code1 > code2) {
                return 1;
            }
            return 0;
        }

    }


    private static class InverseComparator implements Comparator<Integer> {

        public int compare(Integer itg1, Integer itg2) {
            return -itg1.compareTo(itg2);
        }

    }

}
