/* SctServer - Copyright (c) 2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.feed.xml;

import fr.exemole.sctserver.feed.FeedOptions;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.FicheData;
import net.mapeadores.scrutari.data.MotcleData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.util.BaseCheck;
import net.mapeadores.scrutari.db.util.LibelleUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.MotcleNode;
import net.scrutari.datauri.tree.ThesaurusNode;
import net.scrutari.datauri.tree.URITree;


/**
 *
 * @author Vincent Calame
 */
public class TreeEntryContent implements EntryContent {

    private List<BaseBuffer> baseBufferList = new ArrayList<BaseBuffer>();
    private int motcleAddCount = 0;
    private int ficheAddCount = 0;
    private FeedOptions feedOptions;

    public TreeEntryContent(FeedOptions feedOptions) {
        this.feedOptions = feedOptions;
    }

    public int getFicheAddCount() {
        return ficheAddCount;
    }

    public int getMotcleAddCount() {
        return motcleAddCount;
    }

    public void addContent(XMLWriter xmlWriter) throws IOException {
        for (BaseBuffer baseBuffer : baseBufferList) {
            baseBuffer.addContent(xmlWriter, feedOptions);
        }
    }

    void checkUriTree(DataAccess dataAccess, URITree uriTree) {
        boolean withMotscles = !feedOptions.isFichesOnly();
        int baseCount = uriTree.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseNode baseNode = uriTree.getBaseNode(i);
            BaseURI baseURI = baseNode.getBaseURI();
            BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
            if (baseData == null) {
                continue;
            }
            BaseBuffer baseBuffer = new BaseBuffer(baseNode, baseData);
            baseBuffer.checkBase(dataAccess, withMotscles);
            int baseFicheAddCount = baseBuffer.getFicheAddCount();
            int baseMotcleAddCount = baseBuffer.getMotcleAddCount();
            if ((baseFicheAddCount == 0) && (baseMotcleAddCount == 0)) {
                continue;
            }
            ficheAddCount = ficheAddCount + baseFicheAddCount;
            motcleAddCount = motcleAddCount + baseMotcleAddCount;
            baseBufferList.add(baseBuffer);
        }
    }


    private static class BaseBuffer {

        private int motcleAddCount;
        private int ficheAddCount;
        private List<CorpusBuffer> corpusBufferList = new ArrayList<CorpusBuffer>();
        private List<ThesaurusBuffer> thesaurusBufferList = new ArrayList<ThesaurusBuffer>();
        private BaseNode baseNode;
        private BaseData baseData;

        private BaseBuffer(BaseNode baseNode, BaseData baseData) {
            this.baseNode = baseNode;
            this.baseData = baseData;
        }

        private int getFicheAddCount() {
            return ficheAddCount;
        }

        private int getMotcleAddCount() {
            return motcleAddCount;
        }

        private void checkBase(DataAccess dataAccess, boolean withMotscles) {
            motcleAddCount = 0;
            ficheAddCount = 0;
            int corpusCount = baseNode.getCorpusCount();
            for (int j = 0; j < corpusCount; j++) {
                CorpusNode corpusNode = baseNode.getCorpusNode(j);
                CorpusURI corpusURI = corpusNode.getCorpusURI();
                CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
                if (corpusData == null) {
                    continue;
                }
                CorpusBuffer corpusBuffer = new CorpusBuffer(corpusNode, corpusData);
                corpusBuffer.checkFiches(dataAccess);
                int ficheCount = corpusBuffer.getAddCount();
                if (ficheCount == 0) {
                    continue;
                }
                ficheAddCount = ficheAddCount + ficheCount;
                corpusBufferList.add(corpusBuffer);
            }
            if (withMotscles) {
                int thesaurusCount = baseNode.getThesaurusCount();
                for (int j = 0; j < thesaurusCount; j++) {
                    ThesaurusNode thesaurusNode = baseNode.getThesaurusNode(j);
                    ThesaurusURI thesaurusURI = thesaurusNode.getThesaurusURI();
                    ThesaurusData thesaurusData = (ThesaurusData) dataAccess.getScrutariData(thesaurusURI);
                    if (thesaurusData == null) {
                        continue;
                    }
                    ThesaurusBuffer thesaurusBuffer = new ThesaurusBuffer(thesaurusNode, thesaurusData);
                    thesaurusBuffer.checkMotscles(dataAccess);
                    int motcleCount = thesaurusBuffer.getAddCount();
                    if (motcleCount == 0) {
                        continue;
                    }
                    motcleAddCount = motcleAddCount + motcleCount;
                    thesaurusBufferList.add(thesaurusBuffer);
                }
            }
        }

        private void addContent(XMLWriter xmlWriter, FeedOptions feedOptions) throws IOException {
            int workingLangInteger = feedOptions.getWorkingLangInteger();
            BaseCheck baseCheck = BaseCheck.check(baseData, workingLangInteger);
            boolean active = baseNode.isActive();
            xmlWriter.startOpenTag("p", true);
            xmlWriter.addAttribute("class", "SCT_Base");
            xmlWriter.endOpenTag(false);
            insertOpenTag(xmlWriter, active);
            xmlWriter.addText(LibelleUtils.getLibelleString(baseData, BaseData.INTITULE_LONG, workingLangInteger, baseData.getBaseURI().getBaseName()));
            insertCloseTag(xmlWriter, active);
            xmlWriter.closeTag("p", false, false);
            if (!corpusBufferList.isEmpty()) {
                xmlWriter.startOpenTag("ul");
                xmlWriter.addAttribute("class", "SCT_UL_Corpus");
                xmlWriter.endOpenTag();
                for (CorpusBuffer corpusBuffer : corpusBufferList) {
                    corpusBuffer.addContent(xmlWriter, feedOptions, baseCheck);
                }
                xmlWriter.closeTag("ul");
            }
            if (!thesaurusBufferList.isEmpty()) {
                xmlWriter.startOpenTag("ul");
                xmlWriter.addAttribute("class", "SCT_UL_Thesaurus");
                xmlWriter.endOpenTag();
                for (ThesaurusBuffer thesaurusBuffer : thesaurusBufferList) {
                    thesaurusBuffer.addContent(xmlWriter, feedOptions);
                }
                xmlWriter.closeTag("ul");
            }
        }

    }


    private static class CorpusBuffer {

        private List<FicheData> ficheDataList = new ArrayList<FicheData>();
        private CorpusNode corpusNode;
        private CorpusData corpusData;

        private CorpusBuffer(CorpusNode corpusNode, CorpusData corpusData) {
            this.corpusNode = corpusNode;
            this.corpusData = corpusData;
        }

        private int getAddCount() {
            return ficheDataList.size();
        }

        private void checkFiches(DataAccess dataAccess) {
            int ficheCount = corpusNode.getFicheCount();
            for (int k = 0; k < ficheCount; k++) {
                FicheNode ficheNode = corpusNode.getFicheNode(k);
                FicheURI ficheURI = ficheNode.getFicheURI();
                FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                if (ficheData == null) {
                    continue;
                }
                ficheDataList.add(ficheData);
            }
        }

        private void addContent(XMLWriter xmlWriter, FeedOptions feedOptions, BaseCheck baseCheck) throws IOException {
            boolean corpusActive = corpusNode.isActive();
            xmlWriter.openTag("li");
            xmlWriter.startOpenTag("p", true);
            xmlWriter.addAttribute("class", "SCT_Corpus");
            xmlWriter.endOpenTag(false);
            insertOpenTag(xmlWriter, corpusActive);
            xmlWriter.addText(LibelleUtils.getLibelleString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, feedOptions.getWorkingLangInteger(), corpusData.getCorpusURI().getCorpusName()));
            insertCloseTag(xmlWriter, corpusActive);
            xmlWriter.closeTag("p", false, false);
            xmlWriter.startOpenTag("ul");
            xmlWriter.addAttribute("class", "SCT_UL_Fiche");
            xmlWriter.endOpenTag();
            for (FicheData ficheData : ficheDataList) {
                EntryContentUtils.addFicheData(xmlWriter, ficheData, feedOptions, baseCheck);
            }
            xmlWriter.closeTag("ul");
            xmlWriter.closeTag("li");
        }

    }


    private static class ThesaurusBuffer {

        private List<MotcleData> motcleDataList = new ArrayList<MotcleData>();
        private ThesaurusNode thesaurusNode;
        private ThesaurusData thesaurusData;

        private ThesaurusBuffer(ThesaurusNode thesaurusNode, ThesaurusData thesaurusData) {
            this.thesaurusNode = thesaurusNode;
            this.thesaurusData = thesaurusData;
        }

        private int getAddCount() {
            return motcleDataList.size();
        }

        private void checkMotscles(DataAccess dataAccess) {
            int motcleCount = thesaurusNode.getMotcleCount();
            for (int k = 0; k < motcleCount; k++) {
                MotcleNode motcleNode = thesaurusNode.getMotcleNode(k);
                MotcleURI motcleURI = motcleNode.getMotcleURI();
                MotcleData motcleData = (MotcleData) dataAccess.getScrutariData(motcleURI);
                if (motcleData == null) {
                    continue;
                }
                motcleDataList.add(motcleData);
            }
        }

        private void addContent(XMLWriter xmlWriter, FeedOptions feedOptions) throws IOException {
            boolean thesaurusActive = thesaurusNode.isActive();
            xmlWriter.openTag("li");
            xmlWriter.startOpenTag("p", true);
            xmlWriter.addAttribute("class", "SCT_Thesaurus");
            xmlWriter.endOpenTag(false);
            insertOpenTag(xmlWriter, thesaurusActive);
            xmlWriter.addText(LibelleUtils.getLibelleString(thesaurusData, feedOptions.getWorkingLangInteger(), thesaurusData.getThesaurusURI().getThesaurusName()));
            insertCloseTag(xmlWriter, thesaurusActive);
            xmlWriter.closeTag("p", false, false);
            xmlWriter.startOpenTag("ul");
            xmlWriter.addAttribute("class", "SCT_UL_Motcle");
            xmlWriter.endOpenTag();
            for (MotcleData motcleData : motcleDataList) {
                EntryContentUtils.addMotcleData(xmlWriter, motcleData, feedOptions);
            }
            xmlWriter.closeTag("ul");
            xmlWriter.closeTag("li");
        }

    }

    private static void insertOpenTag(XMLWriter xmlWriter, boolean active) throws IOException {
        if (active) {
            xmlWriter.openTag("strong", false, false);
        } else {
            xmlWriter.openTag("em", false, false);
        }
    }

    private static void insertCloseTag(XMLWriter xmlWriter, boolean active) throws IOException {
        if (active) {
            xmlWriter.closeTag("strong", false, false);
        } else {
            xmlWriter.closeTag("em", false, false);
        }
    }

}
