/* SctServer - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import net.mapeadores.scrutari.searchengine.SearchOptionsBuilder;
import net.mapeadores.scrutari.supermotcle.Supermotcle;
import net.mapeadores.scrutari.supermotcle.SupermotcleMap;
import fr.exemole.sctserver.SctServer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.langinteger.LangIntegerUtils;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.TextConstants;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.data.ThesaurusData;
import net.mapeadores.scrutari.db.codefilter.CodeFilterUtils;
import net.mapeadores.scrutari.db.codefilter.CodeficheFilterResult;
import net.mapeadores.scrutari.db.codefilter.MultiCodeCondition;
import net.mapeadores.scrutari.db.codefilter.MultiCodeConditionBuilder;
import net.mapeadores.util.conditions.ConditionConstants;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class RequestMapUtils implements Parameters {

    public static int[] getLangIntegerArray(RequestMap requestMap) {
        String param_langs = requestMap.getParameter(LANG_LIST);
        if (param_langs == null) {
            param_langs = requestMap.getParameter("langs");
        }
        if (param_langs == null) {
            return null;
        }
        int[] langIntegerArray = LangIntegerUtils.toCleanLangIntegerArray(param_langs);
        if (langIntegerArray.length == 0) {
            return null;
        }
        return langIntegerArray;
    }

    public static int getLangUi(RequestMap requestMap, WarningHandler warningHandler) {
        String langUi = requestMap.getParameter(LANG_UI);
        if (langUi == null) {
            return LangInteger.UNDETERMINED;
        }
        try {
            int langInteger = LangInteger.parse(langUi);
            return langInteger;
        } catch (LangIntegerException lie) {
            warningHandler.setCurrentParameter(LANG_UI);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, langUi);
            return LangInteger.UNDETERMINED;
        }
    }

    public static int getLimit(RequestMap requestMap, WarningHandler warningHandler) {
        String limit = requestMap.getParameter(LIMIT);
        if (limit == null) {
            return -1;
        }
        try {
            int limitInteger = Integer.parseInt(limit);
            if (limitInteger > 0) {
                return limitInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(LIMIT);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, limit);
            return -1;
        }
    }

    public static int getStart(RequestMap requestMap, WarningHandler warningHandler) {
        String start = requestMap.getParameter(START);
        if (start == null) {
            return - 1;
        }
        try {
            int globalStartInteger = Integer.parseInt(start);
            if (globalStartInteger > 0) {
                return globalStartInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(START);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, start);
            return -1;
        }
    }

    public static String getSearchResultId(RequestMap requestMap) {
        String identifiant = requestMap.getParameter(Q_ID);
        if (identifiant == null) {
            identifiant = requestMap.getParameter("idrecherche");
        }
        return identifiant;
    }

    public static SearchSequence getSearchSequence(RequestMap requestMap) {
        String q = requestMap.getParameter(Q);
        if (q == null) {
            q = requestMap.getParameter("recherche");
        }
        if (q == null) {
            return null;
        }
        return SearchSequence.createSearchSequence(q);
    }

    public static SearchOptionsBuilder getSearchOptionsBuilder(ScrutariDB scrutariDB, RequestMap requestMap, WarningHandler warningHandler) {
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder(scrutariDB);
        int langUi = getLangUi(requestMap, warningHandler);
        searchOptionsBuilder.setLangUi(langUi);
        int[] langIntegerArray = getLangIntegerArray(requestMap);
        searchOptionsBuilder.setLangIntegerArray(langIntegerArray);
        return searchOptionsBuilder;
    }

    public static Supermotcle getSupermotcle(SctServer sctServer, RequestMap requestMap) {
        String supermotcleString = requestMap.getParameter("supermotcle");
        if (supermotcleString == null) {
            return null;
        }
        int idx = supermotcleString.indexOf("/");
        if (idx == -1) {
            return null;
        }
        int langInteger = 0;
        int code = 0;
        try {
            langInteger = LangInteger.parse(supermotcleString, 0, idx, false);
        } catch (LangIntegerException lie) {
            return null;
        }
        try {
            code = Integer.parseInt(supermotcleString.substring(idx + 1));
        } catch (NumberFormatException nfe) {
            return null;
        }
        SupermotcleMap supermotcleMap = sctServer.getSupermotcleManager().getSupermotcleMap(langInteger);
        if (supermotcleMap == null) {
            return null;
        }
        Supermotcle supermotcle = supermotcleMap.getSupermotcleByCode(code);
        if (supermotcle == null) {
            return null;
        }
        return supermotcle;
    }

    public static MultiCodeCondition getIndexationCodeCondition(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String indexation = requestMap.getParameter(INDEXATION);
        if (indexation == null) {
            return null;
        }
        indexation = indexation.trim();
        if (indexation.length() == 0) {
            return null;
        }
        warningHandler.setCurrentParameter(INDEXATION);
        char firstCarac = indexation.charAt(0);
        short condition = ConditionConstants.MODE_AND;
        if (firstCarac == '|') {
            if (indexation.length() == 1) {
                warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, indexation);
                return CodeFilterUtils.EMPTY_MULTICODECONDTION;
            }
            condition = ConditionConstants.MODE_OR;
            indexation = indexation.substring(1);
        }
        String[] listTokens = StringUtils.getCleanedTokens(indexation, ';');
        int length = listTokens.length;
        if (length == 0) {
            warningHandler.addWarning(WarningKeys.ONLYSEPARATORS_PARAMETER_VALUE, indexation);
            return CodeFilterUtils.EMPTY_MULTICODECONDTION;
        }
        MultiCodeConditionBuilder builder = new MultiCodeConditionBuilder(condition);
        for (int i = 0; i < length; i++) {
            CodeList codeList = toCodeList(dataAccess, listTokens[i], warningHandler, "motcle:", ScrutariDataURI.MOTCLEURI_TYPE);
            if ((codeList != null) && (codeList.getCodeCount() > 0)) {
                builder.add(codeList);
            }
        }
        return builder.toMultiCodeCondition();
    }

    public static RequestInfo getRequestInfo(RequestMap requestMap) {
        boolean withLog = true;
        String param_journal = requestMap.getParameter(LOG);
        if (param_journal == null) {
            param_journal = requestMap.getParameter("journal");
        }
        if ((param_journal != null) && (param_journal.equals("0"))) {
            withLog = false;
        }
        String remote = requestMap.getParameter(REMOTE);
        String site = requestMap.getParameter(SITE);
        String referrer = requestMap.getParameter(REFERRER);
        return new RequestInfo(remote, withLog, site, referrer);
    }

    public static SearchOptions getSearchOptions(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(scrutariDB, requestMap, warningHandler);
        CodeList codecorpusList = getCorpusCodeList(dataAccess, requestMap, warningHandler);
        searchOptionsBuilder.setCodethesaurusList(getThesaurusCodeList(dataAccess, requestMap, warningHandler));
        MultiCodeCondition multiCodeCondition = RequestMapUtils.getIndexationCodeCondition(dataAccess, requestMap, warningHandler);
        if (multiCodeCondition != null) {
            CodeficheFilterResult codeficheFilterResult = CodeFilterUtils.toIndexationCodeficheFilter(dataAccess, multiCodeCondition, codecorpusList);
            searchOptionsBuilder.setCodecorpusList(codeficheFilterResult.getCodecorpusList());
            searchOptionsBuilder.setCodeficheFilter(codeficheFilterResult.getCodeficheFilter());
        } else {
            searchOptionsBuilder.setCodecorpusList(codecorpusList);
        }
        Supermotcle supermotcle = RequestMapUtils.getSupermotcle(sctServer, requestMap);
        if (supermotcle != null) {
            searchOptionsBuilder.setCodemotcleFilterFromSupermotcle(supermotcle);
        }
        searchOptionsBuilder.setDefaultSearchType(TextConstants.STARTSWITH);
        dataAccess.close();
        return searchOptionsBuilder.toSearchOptions();
    }

    public static CodeList getCorpusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(CORPUS_LIST);
        if (param == null) {
            param = requestMap.getParameter("codescorpus");
        }
        return toCodeList(dataAccess, param, warningHandler, "corpus:", ScrutariDataURI.CORPUSURI_TYPE);
    }

    public static CodeList getThesaurusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(THESAURUS_LIST);
        return toCodeList(dataAccess, param, warningHandler, "thesaurus:", ScrutariDataURI.THESAURUSURI_TYPE);
    }

    public static CodeList getBaseCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(BASE_LIST);
        if (param == null) {
            param = requestMap.getParameter("codesbases");
        }
        return toCodeList(dataAccess, param, warningHandler, "base:", ScrutariDataURI.BASEURI_TYPE);
    }

    /**
     * Retourne null si la chaine est nulle ou blanche, si la chaine ne comprend
     * aucun caractère correct, retourne une liste vide
     */
    public static CodeList toCodeList(DataAccess dataAccess, String values, WarningHandler warningHandler, String uriBase, short uriType) {
        if (values == null) {
            return null;
        }
        values = values.trim();
        if (values.length() == 0) {
            return null;
        }
        String[] tokens = StringUtils.getCleanedTokens(values, ',');
        int length = tokens.length;
        if (length == 0) {
            warningHandler.addWarning(WarningKeys.ONLYSEPARATORS_PARAMETER_VALUE, values);
            return CodeListUtils.EMPTY_CODELIST;
        }
        Set<Integer> codeSet = new LinkedHashSet<Integer>();
        for (int i = 0; i < length; i++) {
            int code = toCode(dataAccess, tokens[i], warningHandler, uriBase, uriType);
            if (code != -1) {
                codeSet.add(code);
            }
        }
        return CodeListUtils.toCodeList(codeSet);
    }

    private static int toCode(DataAccess dataAccess, String s, WarningHandler warningHandler, String uriBase, short uriType) {
        ScrutariDataURI scrutariDataURI = null;
        char carac1 = s.charAt(0);
        if ((carac1 >= '0') && (carac1 <= '9')) {
            int idx = s.indexOf('/');
            if (idx == -1) {
                try {
                    int code = Integer.parseInt(s);
                    if (!testCode(dataAccess, code, uriType)) {
                        warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, s);
                        return -1;
                    }
                    return code;
                } catch (NumberFormatException nfe) {
                    warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, s);
                    return -1;
                }
            } else {
                String s1 = s.substring(0, idx);
                try {
                    int code1 = Integer.parseInt(s1);
                    ScrutariDataURI startScrutariDataURI = getParentScrutariDataURI(dataAccess, code1);
                    if (startScrutariDataURI == null) {
                        warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, s1 + " (" + s + ")");
                        return -1;
                    }
                    String s2 = s.substring(idx);
                    String uriString = ScrutariDataURI.typeToSchemeString(uriType) + ':' + startScrutariDataURI.getPath() + s2;
                    try {
                        scrutariDataURI = URIParser.parse(uriString);
                    } catch (URIParseException uriParseException) {
                        warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, s2 + " (" + s + ")");
                        return -1;
                    }
                } catch (NumberFormatException nfe) {
                    warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, s1 + " (" + s + ")");
                    return -1;
                }
            }
        } else {
            boolean isRelative = false;
            int idx = s.indexOf(':');
            String uriString = s;
            if (idx == -1) {
                if (uriBase != null) {
                    isRelative = true;
                    uriString = uriBase + uriString;
                } else {
                    warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, s);
                    return -1;
                }
            }
            try {
                scrutariDataURI = URIParser.parse(uriString);

            } catch (URIParseException uriParseException) {
                warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, s);
                return -1;
            }
            if (scrutariDataURI.getType() != uriType) {
                if (isRelative) {
                    throw new IllegalArgumentException("uriBase and uriType are not of the same type");
                } else {
                    warningHandler.addWarning(WarningKeys.WRONG_SCRUTARIDATATYPE_PARAMETER_VALUE, s);
                    return -1;
                }
            }
        }
        int code = dataAccess.getCode(scrutariDataURI);
        if (code == -1) {
            warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, s);
        }
        return code;
    }

    private static ScrutariDataURI getParentScrutariDataURI(DataAccess dataAccess, int code) {
        BaseData baseData = dataAccess.getBaseDataByCode(code);
        if (baseData != null) {
            return baseData.getBaseURI();
        }
        CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
        if (corpusData != null) {
            return corpusData.getCorpusURI();
        }
        ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
        if (thesaurusData != null) {
            return thesaurusData.getThesaurusURI();
        }
        return null;
    }

    private static boolean testCode(DataAccess dataAccess, int code, short uriType) {
        switch (uriType) {
            case ScrutariDataURI.BASEURI_TYPE:
                return (dataAccess.getBaseDataByCode(code) != null);
            case ScrutariDataURI.CORPUSURI_TYPE:
                return (dataAccess.getCorpusDataByCode(code) != null);
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return (dataAccess.getThesaurusDataByCode(code) != null);
            case ScrutariDataURI.FICHEURI_TYPE:
                return (dataAccess.getFicheDataByCode(code) != null);
            case ScrutariDataURI.MOTCLEURI_TYPE:
                return (dataAccess.getMotcleDataByCode(code) != null);
            default:
                throw new SwitchException("uriType = " + uriType);
        }

    }

    public static IntitulesFilter getIntitulesFilter(RequestMap requestMap) {
        String intitules = requestMap.getParameter(INTITULES);
        if (intitules == null) {
            return null;
        }
        IntitulesFilter intitulesFilter = new IntitulesFilter();
        intitulesFilter.buildFromString(intitules);
        return intitulesFilter;
    }

    public static FicheFieldsFilter getFicheFieldsFilter(RequestMap requestMap) {
        FicheFieldsFilter ficheFieldsFilter = new FicheFieldsFilter();
        String ficheFields = requestMap.getParameter(FICHE_FIELDS);
        if (ficheFields == null) {
            ficheFields = requestMap.getParameter("prop");
        }
        if (ficheFields != null) {
            ficheFieldsFilter.buildFromString(ficheFields);
        }
        return ficheFieldsFilter;
    }

    public static MotcleFieldsFilter getMotcleFieldsFilter(RequestMap requestMap) {
        MotcleFieldsFilter motcleFieldsFilter = new MotcleFieldsFilter();
        String ficheFields = requestMap.getParameter(MOTCLE_FIELDS);
        if (ficheFields != null) {
            motcleFieldsFilter.buildFromString(ficheFields);
        }
        return motcleFieldsFilter;
    }

    public static CountFilter getCountFilter(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        CountFilter countFilter = new CountFilter();
        countFilter.setLangIntegerArray(getLangIntegerArray(requestMap));
        countFilter.setCorpusCodeList(getCorpusCodeList(dataAccess, requestMap, warningHandler));
        countFilter.setBaseCodeList(getBaseCodeList(dataAccess, requestMap, warningHandler));
        return countFilter;
    }

}
