/* SctServer - Copyright (c) 2010-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.request.CountFilter;
import fr.exemole.sctserver.request.ErrorKeys;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.IntitulesFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import fr.exemole.sctserver.request.Parameters;
import fr.exemole.sctserver.request.RequestInfo;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.db.explore.ExploreUtils;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.MotcleSearchResult;
import net.mapeadores.scrutari.searchengine.SearchOptions;
import net.mapeadores.scrutari.searchengine.SearchOptionsBuilder;
import net.mapeadores.scrutari.searchengine.SupermotcleSearchResult;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.SearchSequence;
import net.scrutari.datauri.ScrutariDataURI;


/**
 *
 * @author Vincent Calame
 */
public class JsonProducerFactory implements Parameters {

    public static JsonProducer getJsonProducer(SctServer sctServer, RequestMap requestMap) {
        WarningHandler warningHandler = new WarningHandler();
        AbstractJsonProducer abstractJsonProducer = getJsonProducer(sctServer, requestMap, warningHandler);
        if (!warningHandler.isEmpty()) {
            String warnings = requestMap.getParameter(WARNINGS);
            if ((warnings != null) && (warnings.equals("1"))) {
                abstractJsonProducer.setWarningHandler(warningHandler);
            }
        }
        return abstractJsonProducer;
    }

    private static AbstractJsonProducer getJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        String type = requestMap.getParameter(TYPE);
        if (type == null) {
            return getErrorJsonProducer(ErrorKeys.MISSING_PARAMETER, TYPE);
        }
        if (type.isEmpty()) {
            return getErrorJsonProducer(ErrorKeys.EMPTY_PARAMETER, TYPE);
        }
        if (type.equals("motcle")) {
            return getMotcleDataJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("q-motcle")) {
            return getMotcleSearchResultJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("q-supermotcle")) {
            return getSupermotcleSearchResultJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("base")) {
            return getBaseDataJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("fiche")) {
            return getFicheDataJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("q-fiche")) {
            return getFicheSearchResultJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("corpus")) {
            return getCorpusDataJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("thesaurus")) {
            return getThesaurusDataJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("c-fiche")) {
            return getFicheCountJsonProducer(sctServer, requestMap, warningHandler);
        } else if (type.equals("classe")) {
            return getClasseDefJsonProducer(sctServer, requestMap, warningHandler);
        } else {
            return getErrorJsonProducer(ErrorKeys.UNKNOWN_PARAMETER_VALUE, TYPE, type);
        }
    }

    private static AbstractJsonProducer getSupermotcleSearchResultJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        SearchSequence searchSequence = RequestMapUtils.getSearchSequence(requestMap);
        if (searchSequence == null) {
            return getErrorJsonProducer(ErrorKeys.MISSING_PARAMETER, Q);
        }
        String languiString = requestMap.getParameter(LANG_UI);
        if (languiString == null) {
            return getErrorJsonProducer(ErrorKeys.MISSING_PARAMETER, LANG_UI);
        }
        if (languiString.length() == 0) {
            return getErrorJsonProducer(ErrorKeys.EMPTY_PARAMETER, LANG_UI);
        }
        int langui = LangInteger.UNDETERMINED;
        try {
            langui = LangInteger.parse(languiString);
        } catch (LangIntegerException lie) {
            return getErrorJsonProducer(ErrorKeys.MALFORMED_PARAMETER_VALUE, LANG_UI, languiString);
        }
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(scrutariDB, requestMap, warningHandler);
        int[] tmp = new int[1];
        tmp[0] = langui;
        searchOptionsBuilder.setLangIntegerArray(tmp);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        SupermotcleSearchResult searchResult = sctServer.newSupermotcleSearch(searchSequence, searchOptionsBuilder.toSearchOptions());
        return new SupermotcleSearchResultJsonProducer(searchResult, sctServer, langui, limit);
    }

    private static AbstractJsonProducer getMotcleSearchResultJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        SearchSequence searchSequence = RequestMapUtils.getSearchSequence(requestMap);
        if (searchSequence == null) {
            return getErrorJsonProducer(ErrorKeys.MISSING_PARAMETER, Q);
        }
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        SearchOptionsBuilder searchOptionsBuilder = RequestMapUtils.getSearchOptionsBuilder(scrutariDB, requestMap, warningHandler);
        if (searchOptionsBuilder.getLangIntegerArray() == null) {
            //@todo message d'erreur
            return null;
        }
        int langInteger = checkUniquelang(searchOptionsBuilder);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        MotcleSearchResult searchResult = sctServer.newMotcleSearch(searchSequence, searchOptionsBuilder.toSearchOptions());
        return new MotcleSearchResultJsonProducer(searchResult, scrutariDB, langInteger, limit);

    }

    private static AbstractJsonProducer getFicheSearchResultJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        String searchResultId = RequestMapUtils.getSearchResultId(requestMap);
        FicheSearchResult searchResult = null;
        if (searchResultId != null) {
            if (searchResultId.isEmpty()) {
                return getErrorJsonProducer(ErrorKeys.EMPTY_PARAMETER, Q_ID);
            }
            searchResult = sctServer.getSearchResult(searchResultId);
            if (searchResult == null) {
                return getErrorJsonProducer(ErrorKeys.UNKNOWN_PARAMETER_VALUE, Q_ID, searchResultId);
            }
        } else {
            SearchSequence searchSequence = RequestMapUtils.getSearchSequence(requestMap);
            if (searchSequence == null) {
                return getErrorJsonProducer(ErrorKeys.MISSING_PARAMETER, Q);
            }
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(sctServer, requestMap, warningHandler);
            RequestInfo requestInfo = RequestMapUtils.getRequestInfo(requestMap);
            searchResult = sctServer.newSearch(searchSequence, searchOptions, requestInfo);
        }
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        IntitulesFilter intitulesFilter = RequestMapUtils.getIntitulesFilter(requestMap);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        int start = RequestMapUtils.getStart(requestMap, warningHandler);
        return new FicheSearchResultJsonProducer(searchResult, sctServer.getScrutariDB(), sctServer.getOptionsManager().getGlobalSearchOptions(), ficheFieldsFilter, motcleFieldsFilter, intitulesFilter, limit, start);
    }

    private static AbstractJsonProducer getClasseDefJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new ClasseDefJsonProducer(sctServer, langUi);
    }

    private static AbstractJsonProducer getBaseDataJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new BaseDataJsonProducer(sctServer.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getCorpusDataJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new CorpusDataJsonProducer(sctServer.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getThesaurusDataJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        return new ThesaurusDataJsonProducer(sctServer.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getFicheDataJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        int langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList codeficheList;
        String ficheListParam = requestMap.getParameter(FICHE_LIST);
        if (ficheListParam != null) {
            codeficheList = RequestMapUtils.toCodeList(dataAccess, ficheListParam, warningHandler, "fiche:", ScrutariDataURI.FICHEURI_TYPE);
            if (codeficheList == null) {
                codeficheList = CodeListUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(sctServer, requestMap, warningHandler);
            codeficheList = ExploreUtils.getCodeficheList(dataAccess, searchOptions.getCodeficheFilter(), searchOptions.getCodecorpusList());
        }
        dataAccess.close();
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        return new FicheDataJsonProducer(sctServer.getScrutariDB(), codeficheList, ficheFieldsFilter, langUi);
    }

    private static AbstractJsonProducer getMotcleDataJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList codemotcleList;
        String motcleListParam = requestMap.getParameter(MOTCLE_LIST);
        if (motcleListParam != null) {
            codemotcleList = RequestMapUtils.toCodeList(dataAccess, motcleListParam, warningHandler, "motcle:", ScrutariDataURI.MOTCLEURI_TYPE);
            if (codemotcleList == null) {
                codemotcleList = CodeListUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(sctServer, requestMap, warningHandler);
            codemotcleList = ExploreUtils.getCodemotcleList(dataAccess, searchOptions.getCodemotcleFilter(), searchOptions.getCodethesaurusList());
        }
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        return new MotcleDataJsonProducer(sctServer.getScrutariDB(), codemotcleList, motcleFieldsFilter);
    }

    private static int checkUniquelang(SearchOptionsBuilder searchOptionsBuilder) {
        int[] langIntegerArray = searchOptionsBuilder.getLangIntegerArray();
        if (langIntegerArray.length > 1) {
            int[] tmp = new int[1];
            tmp[0] = langIntegerArray[0];
            langIntegerArray = tmp;
            searchOptionsBuilder.setLangIntegerArray(tmp);
        }
        return langIntegerArray[0];
    }

    private static AbstractJsonProducer getFicheCountJsonProducer(SctServer sctServer, RequestMap requestMap, WarningHandler warningHandler) {
        DataAccess dataAccess = sctServer.getScrutariDB().openDataAccess();
        CountFilter countFilter = RequestMapUtils.getCountFilter(dataAccess, requestMap, warningHandler);
        dataAccess.close();
        return new FicheCountJsonProducer(sctServer.getScrutariDB(), countFilter);
    }

    private static AbstractJsonProducer getErrorJsonProducer(String errorKey, String parameter) {
        return new ErrorJsonProducer(errorKey, parameter, null);
    }

    private static AbstractJsonProducer getErrorJsonProducer(String errorKey, String parameter, String value) {
        return new ErrorJsonProducer(errorKey, parameter, value);
    }

}
