/* SctServer - Copyright (c) 2005-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.xml;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.request.RequestInfo;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.scrutari.data.BaseData;
import net.mapeadores.scrutari.data.CorpusData;
import net.mapeadores.scrutari.db.CodeList;
import net.mapeadores.scrutari.db.CodeListUtils;
import net.mapeadores.scrutari.db.DataAccess;
import net.mapeadores.scrutari.db.ScrutariDB;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.SearchOptionsBuilder;
import net.mapeadores.scrutari.searchengine.util.SearchOptionsUtils;
import net.mapeadores.util.collections.LinkedIntHashMap;
import net.mapeadores.util.langinteger.LangInteger;
import net.mapeadores.util.langinteger.LangIntegerException;
import net.mapeadores.util.text.SearchSequence;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 *
 * @author Vincent Calame
 */
public class SearchRequest {

    private final static int DEFAULT_LANG = LangInteger.build("fr");
    private SearchSequence searchSequence;
    private FicheSearchResult searchResult;

    /**
     * Construit une nouvelle instance à partir du paramètre xml
     */
    public SearchRequest(String xml, SctServer sctServer, RequestInfo requestInfo) throws XmlRequestException {
        if (xml == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_XML_PARAMETER);
        }
        LinkedIntHashMap baseCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap corpusCodeMap = new LinkedIntHashMap();
        LinkedIntHashMap langIntMap = new LinkedIntHashMap();
        int langUi = DEFAULT_LANG;
        ScrutariDB scrutariDB = sctServer.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        String source = null;
        Document doc = XmlRequestUtils.getDocument(xml);
        Element racine = doc.getDocumentElement();
        NodeList nodeList = racine.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node instanceof Element) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("source")) {
                    source = XMLUtils.getData(el);
                } else if (tagname.equals("base")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("base-code"));
                        baseCodeMap.put(code, Boolean.TRUE);
                    } catch (NumberFormatException nfe) {
                    }
                } else if (tagname.equals("corpus")) {
                    try {
                        int code = Integer.parseInt(el.getAttribute("corpus-code"));
                        CorpusData corpusInfo = dataAccess.getCorpusDataByCode(code);
                        if (corpusInfo != null) {
                            corpusCodeMap.put(code, Boolean.TRUE);
                        }
                    } catch (NumberFormatException nfe) {
                    }
                } else if (tagname.equals("lang")) {
                    String strg_lang = el.getAttribute("lang");
                    try {
                        int langInteger = LangInteger.parse(strg_lang);
                        langIntMap.put(langInteger, Boolean.TRUE);
                    } catch (LangIntegerException ile) {
                    }
                } else if (tagname.equals("lang-ui")) {
                    String strg_langui = el.getAttribute("lang");
                    try {
                        langUi = LangInteger.parse(strg_langui);
                    } catch (LangIntegerException ile) {
                    }
                }
            }
        }
        for (int i = 0; i < baseCodeMap.size(); i++) {
            BaseData baseInfo = dataAccess.getBaseDataByCode(baseCodeMap.getKeyAt(i));
            if (baseInfo != null) {
                int count = baseInfo.getCorpusCount();
                for (int j = 0; j < count; j++) {
                    corpusCodeMap.put(baseInfo.getCodeCorpus(j), Boolean.TRUE);
                }
            }
        }
        if (source == null) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.MISSING_SOURCE_ELEMENT);
        }
        searchSequence = SearchSequence.createSearchSequence(source);
        if (searchSequence.isEmpty()) {
            throw new XmlRequestException(XmlRequestException.CLIENT_SOURCE, XmlRequestException.POOR_SOURCE_ELEMENT, searchSequence.getSource());
        }
        CodeList codecorpusList = null;
        if (corpusCodeMap.size() > 0) {
            codecorpusList = CodeListUtils.toCodeList(corpusCodeMap.getKeyArray());
        }
        int[] langIntegerArray = (langIntMap.size() > 0) ? langIntMap.getKeyArray() : null;
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder(scrutariDB);
        searchOptionsBuilder.setLangUi(langUi);
        searchOptionsBuilder.setLangIntegerArray(langIntegerArray);
        searchOptionsBuilder.setCodecorpusList(codecorpusList);
        if (codecorpusList != null) {
            searchOptionsBuilder.setCodethesaurusList(SearchOptionsUtils.getSelectionThesaurusCodeList(scrutariDB.getCountStats(), dataAccess, codecorpusList));
        }
        searchOptionsBuilder.setDefaultSearchType(TextConstants.STARTSWITH);
        searchResult = sctServer.newSearch(searchSequence, searchOptionsBuilder.toSearchOptions(), requestInfo);
        dataAccess.close();
    }

    /**
     * Écrit la réponse à la requête : il s'agit de l'arbre xml « search ».
     */
    public void printResponse(HttpServletResponse reponse) throws IOException {
        int allMotcleCount = searchResult.getMotcleCount();
        int completeMotcleCount = searchResult.getCompleteMotcleCount();
        int tabulations = 0;
        reponse.setContentType("text/xml;charset=UTF-8");
        PrintWriter pw = reponse.getWriter();
        pw.print("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<!DOCTYPE search SYSTEM \"dtd/search.dtd\">");
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<search id=\"search\" search-id=\"");
        pw.print(searchResult.getIdentifiant());
        pw.print("\" fiche-count=\"");
        pw.print(String.valueOf(searchResult.getFicheCount()));
        pw.print("\" fiche-maximum=\"");
        pw.print(String.valueOf(searchResult.getFicheMaximum()));
        pw.print("\" all-motcle-count=\"");
        pw.print(String.valueOf(allMotcleCount));
        pw.print("\" complete-motcle-count=\"");
        pw.print(String.valueOf(completeMotcleCount));
        pw.print("\" partial-motcle-count=\"");
        pw.print(String.valueOf(allMotcleCount - completeMotcleCount));
        pw.print("\"");
        pw.print(">");
        printSequence(pw, searchSequence, tabulations + 1);
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</search>");

    }

    /**
     * Écrit le format xml décrivant la séquence.
     */
    public void printSequence(PrintWriter pw, SearchSequence searchSequence, int tabulations) {
        XMLUtils.printTabs(pw, tabulations);
        pw.print("<sequence id=\"sequence\" token-count=\"");
        pw.print(String.valueOf(searchSequence.getTokenCount()));
        pw.print("\">");
        XMLUtils.printTabs(pw, tabulations + 1);
        pw.print("<source id=\"source\" clean=\"");
        XMLUtils.printEscape(pw, searchSequence.toString());
        pw.print("\">");
        XMLUtils.printEscape(pw, searchSequence.getSource());
        pw.print("</source>");
        for (int i = 0; i < searchSequence.getTokenCount(); i++) {
            SearchSequence.Token token = searchSequence.getToken(i);
            XMLUtils.printTabs(pw, tabulations + 1);
            if (token.isMultiToken()) {
                pw.print("<token token-type=\"multi\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\">");
                SearchSequence.SimpleToken[] array = ((SearchSequence.MultiToken) token).getSubtokenArray();
                for (int j = 0; j < array.length; j++) {
                    SearchSequence.SimpleToken simpleToken = array[j];
                    XMLUtils.printTabs(pw, tabulations + 2);
                    pw.print("<subtoken search-type=\"");
                    pw.print(TextConstants.toString(simpleToken.getSearchType()));
                    pw.print("\">");
                    XMLUtils.printEscape(pw, simpleToken.toString(false));
                    pw.print("</subtoken>");
                }
                XMLUtils.printTabs(pw, tabulations + 1);
                pw.print("</token>");
            } else {
                SearchSequence.SimpleToken simpleToken = (SearchSequence.SimpleToken) token;
                pw.print("<token token-type=\"simple\" id=\"token_");
                pw.print(String.valueOf(i));
                pw.print("\" token-index=\"");
                pw.print(String.valueOf(i));
                pw.print("\" search-type=\"");
                pw.print(TextConstants.toString(simpleToken.getSearchType()));
                pw.print("\">");
                XMLUtils.printEscape(pw, simpleToken.toString(false));
                pw.print("</token>");
            }
        }
        XMLUtils.printTabs(pw, tabulations);
        pw.print("</sequence>");
    }

}
