/* SctServer_Directory - Copyright (c) 2008-2012 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory;

import fr.exemole.sctserver.SctServer;
import fr.exemole.sctserver.SearchLog;
import fr.exemole.sctserver.SearchLog.DayLog;
import fr.exemole.sctserver.admin.ScrutariInfo;
import fr.exemole.sctserver.conf.ConfDirectory;
import fr.exemole.sctserver.conf.DefaultConfDirectory;
import fr.exemole.sctserver.directory.cache.DirectoryDataCacheWriter;
import fr.exemole.sctserver.directory.cache.DirectoryLexieCacheWriter;
import fr.exemole.sctserver.searchlog.DefaultQueryLog;
import fr.exemole.sctserver.storage.LogAccess;
import fr.exemole.sctserver.storage.SctServerStorage;
import fr.exemole.sctserver.storage.SctStorageException;
import fr.exemole.sctserver.storage.SearchLogAccess;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.mapeadores.scrutari.db.CountStats;
import net.mapeadores.scrutari.db.DataCacheWriter;
import net.mapeadores.scrutari.db.LexieCacheWriter;
import net.mapeadores.scrutari.db.ScrutariDBName;
import net.mapeadores.scrutari.searchengine.FicheSearchResult;
import net.mapeadores.scrutari.searchengine.io.FicheSearchResultStorage;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.io.MimeTypeConstants;
import net.mapeadores.util.io.ResourceInfo;
import net.mapeadores.util.io.ResourceInfoUtils;
import net.mapeadores.util.io.docstream.FileDocStream;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.mapeadores.util.text.StringUtils;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.datauri.tree.io.TsvReader;
import net.scrutari.datauri.tree.io.TsvWriter;


/**
 *
 * @author Vincent Calame
 */
public class SctServerDirectory implements SctServerStorage {

    private final static int LOG_VERSION = 1;
    private final static DateFormat dateformatanneemois = new SimpleDateFormat("yyyyMM");
    private final static DateFormat dateformatjour = new SimpleDateFormat("dd");
    private File varDir;
    private File cacheDirectory;
    private File berkeleyDirectory;
    private File logsDirectory;
    private File searchresultsDirectory;
    private File dataDirectory;
    private File uriDirectory;
    private InternalLogAccess internalLogAccess = new InternalLogAccess();
    private InternalSearchLogAccess searchLogAccess = new InternalSearchLogAccess();
    private InternalSearchLog searchLog = new InternalSearchLog();
    private DefaultConfDirectory confDirectory;

    public SctServerDirectory(File confDir, File varDir) {
        this.varDir = varDir;
        confDirectory = new DefaultConfDirectory(confDir);
        berkeleyDirectory = getDirectory("berkeley");
        cacheDirectory = getDirectory("cache");
        try {
            IOUtils.cleanDirectory(berkeleyDirectory);
            IOUtils.cleanDirectory(cacheDirectory);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        logsDirectory = getDirectory("logs");
        searchresultsDirectory = getDirectory("searchresults");
        dataDirectory = getDirectory("data");
        uriDirectory = getDirectory("uri");

        File jeLock = new File(berkeleyDirectory, "je.lck");
        if (jeLock.exists()) {
            jeLock.delete();
        }
    }

    public DataCacheWriter getDataCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, "scrutaridb_" + scrutariDbName.getName());
        dbCacheDirectory.mkdirs();
        return new DirectoryDataCacheWriter(dbCacheDirectory);
    }

    public LexieCacheWriter getLexieCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, "scrutaridb_" + scrutariDbName.getName());
        dbCacheDirectory.mkdirs();
        return new DirectoryLexieCacheWriter(dbCacheDirectory);
    }

    public void clearCache(ScrutariDBName scrutariDbName) {
        try {
            File directory = new File(berkeleyDirectory, "scrutaridb_" + scrutariDbName.getName());
            if (directory.exists()) {
                IOUtils.cleanDirectory(directory);
                directory.delete();
            }
            File dbCacheDirectory = new File(cacheDirectory, "scrutaridb_" + scrutariDbName.getName());
            if (dbCacheDirectory.exists()) {
                IOUtils.cleanDirectory(dbCacheDirectory);
                dbCacheDirectory.delete();
            }
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public synchronized void saveFicheSearchResult(FicheSearchResult ficheSearchResult) {
        try {
            String identifiant = ficheSearchResult.getIdentifiant();
            int idx = identifiant.indexOf("-");
            File sousdir = new File(searchresultsDirectory, identifiant.substring(0, idx));
            sousdir.mkdirs();
            File f = new File(sousdir, "r" + identifiant.substring(idx + 1));
            DataOutputStream dataOutput = new DataOutputStream(new FileOutputStream(f));
            DataOutputPrimitivesWriter primWriter = new DataOutputPrimitivesWriter(dataOutput);
            FicheSearchResultStorage.write(primWriter, ficheSearchResult);
            dataOutput.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public LogAccess getLogAccess() {
        return internalLogAccess;
    }

    public ConfDirectory getConfDirectory() {
        return confDirectory;
    }

    public FicheSearchResult getFicheSearchResult(String identifiant) {
        FicheSearchResult searchResult = null;
        try {
            int idx = identifiant.indexOf("-");
            if (idx == -1) {
                return null;
            }
            File f = new File(searchresultsDirectory, identifiant.substring(0, idx) + File.separator + "r" + identifiant.substring(idx + 1));
            if (!f.exists()) {
                return null;
            }
            DataInputStream dataInput = new DataInputStream(new FileInputStream(f));
            DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataInput);
            searchResult = FicheSearchResultStorage.read(primReader);
            dataInput.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        return searchResult;
    }

    private File getDirectory(String name) {
        File directory = new File(varDir, name);
        testDirectory(directory);
        return directory;
    }

    private void testDirectory(File directory) {
        if (!directory.exists()) {
            directory.mkdirs();
        } else if (!directory.isDirectory()) {
            throw new SctServerDirectoryException(SctServerDirectoryException.NOT_A_DIRECTORY, directory.getPath());
        }
    }

    public SearchLogAccess getSearchLogAccess() {
        return searchLogAccess;
    }

    public void clean() {
        try {
            cleanDirectory(berkeleyDirectory);
            cleanDirectory(cacheDirectory);
            cleanDirectory(logsDirectory);
            cleanDirectory(searchresultsDirectory);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public InputStream getScrutariData(ScrutariInfo scrutariInfo) {
        File file = getDataFile(scrutariInfo);
        if (!file.exists()) {
            return null;
        }
        if (file.isDirectory()) {
            return null;
        }
        try {
            return new FileInputStream(file);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public void saveScrutariData(ScrutariInfo scrutariInfo, InputStream inputStream) {
        File file = getDataFile(scrutariInfo);
        try {
            OutputStream outputStream = new FileOutputStream(file);
            IOUtils.copy(inputStream, outputStream);
            outputStream.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    private File getDataFile(ScrutariInfo sourceInfo) {
        return new File(dataDirectory, sourceInfo.getKeyString() + ".xml");
    }

    public void cleanSearchResults() {
        long time = System.currentTimeMillis();
        long past = time - (1000 * 60 * 60 * 48);
        File[] files = searchresultsDirectory.listFiles();
        for (int i = 0; i < files.length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                long lastModified = f.lastModified();
                if (lastModified < past) {
                    try {
                        IOUtils.deleteDirectory(f);
                    } catch (IOException ioe) {
                        throw new SctStorageException(ioe);
                    }
                }
            }
        }
    }

    public void saveURITree(URITree uriTree, ScrutariDBName scrutariDBName) {
        File file = getUriTreeFile(scrutariDBName);
        try {
            BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"));
            TsvWriter.write(writer, uriTree);
            writer.close();
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public URITree getURITree(Date date) {
        ScrutariDBName name = null;
        if (date != null) {
            name = ScrutariDBName.newName(date);
        }
        File file = getUriTreeFile(name);
        if (!file.exists()) {
            return null;
        }
        try {
            InputStreamReader reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
            URITree uriTree = TsvReader.readURITree(reader);
            reader.close();
            return uriTree;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    public Date[] getAddURITreeDateArray() {
        File dir = new File(uriDirectory, "add");
        if (!dir.exists()) {
            return new Date[0];
        }
        Set<Date> set = new TreeSet<Date>();
        File[] files = dir.listFiles();
        int length = files.length;
        for (int i = 0; i < length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (name.endsWith(".txt")) {
                name = name.substring(0, name.length() - 4);
                try {
                    Date date = ScrutariDBName.parseNameFormat(name);
                    set.add(date);
                } catch (ParseException pe) {
                }
            }
        }
        int size = set.size();
        int p = size - 1;
        Date[] result = new Date[size];
        for (Date date : set) {
            result[p] = date;
            p--;
        }
        return result;
    }

    private File getUriTreeFile(ScrutariDBName scrutariDBName) {
        if (scrutariDBName == null) {
            return new File(uriDirectory, "tree.txt");
        }
        File dir = new File(uriDirectory, "add");
        dir.mkdir();
        return new File(dir, scrutariDBName.getName() + ".txt");
    }

    private void cleanDirectory(File directory) throws IOException {
        IOUtils.deleteDirectory(directory);
        directory.mkdirs();
    }


    private class InternalLogAccess implements LogAccess {

        public DocStream getLogDocStream(String logName) {
            File f = new File(logsDirectory, logName);
            if (!f.exists()) {
                return null;
            }
            if (f.isDirectory()) {
                return null;
            }
            FileDocStream docStream = new FileDocStream(f);
            docStream.setMimeType(MimeTypeConstants.PLAIN);
            return docStream;
        }

        public OutputStream getLogOutputStream(String logName) {
            File f = new File(logsDirectory, logName);
            try {
                return new FileOutputStream(f);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        public ResourceInfo[] getCurrentLogArray() {
            return ResourceInfoUtils.toResourceInfoArray(logsDirectory);
        }

    }


    private class InternalSearchLogAccess implements SearchLogAccess {

        public void log(SctServer sctServer, SearchLog.QueryLog queryLog) {
            Date date = new Date();
            String anneemois = dateformatanneemois.format(date);
            String jour = dateformatjour.format(date);
            File repjournal = new File(logsDirectory, anneemois);
            File logFile = new File(repjournal, jour);
            StringBuilder buf = new StringBuilder();
            if (!logFile.exists()) {
                CountStats countStats = sctServer.getScrutariDB().getCountStats();
                append(buf, "fiche", countStats.getFicheCount());
                append(buf, "motcle", countStats.getMotcleCount());
                append(buf, "corpus", countStats.getCorpusCount());
                append(buf, "thesaurus", countStats.getThesaurusCount());
                append(buf, "base", countStats.getBaseCount());
                append(buf, "lexie", countStats.getLexieCount());
                append(buf, "indexation", countStats.getIndexationCount());
                append(buf, "version", LOG_VERSION);
            }
            buf.append(queryLog.getSearchSequence());
            String site = queryLog.getSite();
            buf.append(':');
            buf.append(site);
            appendInt(buf, queryLog.getFicheCount());
            appendInt(buf, queryLog.getFicheMaximum());
            appendInt(buf, queryLog.getAllMotcleCount());
            appendInt(buf, queryLog.getCompleteMotcleCount());
            buf.append('\n');
            try {
                repjournal.mkdirs();
                synchronized (this) {
                    Writer writer = new OutputStreamWriter(new FileOutputStream(logFile, true), "UTF-8");
                    writer.write(buf.toString());
                    writer.close();
                }
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

        public SearchLog getSearchLog() {
            return searchLog;
        }

        private void appendInt(StringBuilder buf, int value) {
            buf.append(':');
            if (value >= 0) {
                buf.append(value);
            }
        }

        private void append(StringBuilder buf, String type, int value) {
            buf.append("!");
            buf.append(type);
            buf.append("=");
            buf.append(value);
            buf.append('\n');
        }

    }


    private class InternalSearchLog implements SearchLog {

        public int[] getMonthIntegerArray() {
            int[] result;
            if ((!logsDirectory.exists()) || (!logsDirectory.isDirectory())) {
                return new int[0];
            }
            File[] liste = logsDirectory.listFiles();
            SortedSet<Integer> monthSet = new TreeSet<Integer>();
            for (int i = 0; i < liste.length; i++) {
                File dir = liste[i];
                if (!dir.isDirectory()) {
                    continue;
                }
                String name = dir.getName();
                if (name.length() != 6) {
                    continue;
                }
                String dateString = name.substring(0, 4) + "-" + name.substring(4);
                try {
                    int mois = DateInteger.parse(dateString);
                    monthSet.add(new Integer(mois));
                } catch (DateIntegerException ide) {
                    continue;
                }

            }
            int size = monthSet.size();
            result = new int[size];
            int i = size - 1;
            for (Integer itg : monthSet) {
                result[i] = itg.intValue();
                i--;
            }
            return result;
        }

        public DayLog[] getDayLogArray(int monthInteger) {
            SortedMap<String, InternalDayLog> tree = new TreeMap<String, InternalDayLog>();
            InternalDayLog[] result;
            String moisString = DateInteger.toString(monthInteger);
            File repmois = new File(logsDirectory, moisString.substring(0, 4) + moisString.substring(5));
            if ((!repmois.exists()) || (!repmois.isDirectory())) {
                return new InternalDayLog[0];
            }
            File[] liste = repmois.listFiles();
            for (int j = 0; j < liste.length; j++) {
                File fich = liste[j];
                if (!fich.isDirectory()) {
                    String nom = fich.getName();
                    if (nom.length() == 2) {
                        int dateInteger = monthInteger;
                        try {
                            int jour = DateInteger.parse(moisString + "-" + nom);
                            InternalDayLog dayLog = new InternalDayLog(jour);
                            dayLog.init(fich);

                            tree.put(nom, dayLog);
                        } catch (DateIntegerException ide) {
                        }

                    }
                }
            }
            result = new InternalDayLog[tree.size()];
            int i = 0;
            for (InternalDayLog dayLog : tree.values()) {
                result[i] = dayLog;
                i++;
            }
            return result;
        }

    }


    private class InternalDayLog implements SearchLog.DayLog {

        private int dateInteger;
        private InternalQueryLog[] queryLogArray = new InternalQueryLog[0];
        private InternalDayStats dayStats = new InternalDayStats();
        private int version = 0;

        InternalDayLog(int dateInteger) {
            this.dateInteger = dateInteger;
        }

        public int getDateInteger() {
            return dateInteger;
        }

        public SearchLog.QueryLog[] getQueryLogArray() {
            return queryLogArray;
        }

        public SearchLog.DayStats getDayStats() {
            return dayStats;
        }

        private void init(File f) {
            List<InternalQueryLog> list = new ArrayList<InternalQueryLog>();
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(f), "UTF-8"));
                String chaine;
                while ((chaine = reader.readLine()) != null) {
                    chaine = chaine.trim();
                    if (chaine.length() == 0) {
                        continue;
                    }
                    if (chaine.charAt(0) == '!') {
                        initValue(chaine);
                        continue;
                    }
                    InternalQueryLog queryLog = new InternalQueryLog();
                    String[] tokens = StringUtils.getTokens(chaine, ':', StringUtils.EMPTY_INCLUDE);
                    if (version == 0) {
                        initVersion0(queryLog, tokens);
                    } else {
                        initVersion1(queryLog, tokens);
                    }
                    list.add(queryLog);
                }
                reader.close();
            } catch (IOException ioe) {
            }
            int size = list.size();
            queryLogArray = new InternalQueryLog[size];
            for (int i = 0; i < size; i++) {
                queryLogArray[i] = list.get(i);
            }
        }

        /**
         * Version 0 d'enregistrement des journaux, il y avait encore l'adresse IP
         * ou la page d'origine
         */
        private void initVersion0(InternalQueryLog queryLog, String[] tokens) {
            int length = tokens.length;
            for (int i = 0; i < length; i++) {
                String token = tokens[i];
                boolean hasToBreak = false;
                switch (i) {
                    case 0:
                        break;
                    case 1:
                        queryLog.setSearchSequence(token);
                        break;
                    case 2:
                        queryLog.setSite(token);
                        break;
                    case 3:
                        break;
                    case 4:
                        queryLog.setFicheCount(getInteger(token));
                        break;
                    case 5:
                        queryLog.setFicheMaximum(getInteger(token));
                        break;
                    case 6:
                        queryLog.setAllMotcleCount(getInteger(token));
                        break;
                    case 7:
                        queryLog.setCompleteMotcleCount(getInteger(token));
                        break;
                    default:
                        hasToBreak = true;
                }
                if (hasToBreak) {
                    break;
                }
            }
        }

        private void initVersion1(InternalQueryLog queryLog, String[] tokens) {
            int length = tokens.length;
            for (int i = 0; i < length; i++) {
                String token = tokens[i];
                boolean hasToBreak = false;
                switch (i) {
                    case 0:
                        queryLog.setSearchSequence(token);
                        break;
                    case 1:
                        queryLog.setSite(token);
                        break;
                    case 2:
                        queryLog.setFicheCount(getInteger(token));
                        break;
                    case 3:
                        queryLog.setFicheMaximum(getInteger(token));
                        break;
                    case 4:
                        queryLog.setAllMotcleCount(getInteger(token));
                        break;
                    case 5:
                        queryLog.setCompleteMotcleCount(getInteger(token));
                        break;
                    default:
                        hasToBreak = true;
                }
                if (hasToBreak) {
                    break;
                }
            }
        }

        private void initValue(String s) {
            int idx = s.indexOf('=');
            if (idx == -1) {
                return;
            }
            String key = s.substring(1, idx).trim();
            String value = s.substring(idx + 1).trim();
            int val;
            try {
                val = Integer.parseInt(value);
            } catch (NumberFormatException nfe) {
                return;
            }
            if (key.equals("fiche")) {
                dayStats.setFicheCount(val);
            } else if (key.equals("motcle")) {
                dayStats.setMotcleCount(val);
            } else if (key.equals("corpus")) {
                dayStats.setCorpusCount(val);
            } else if (key.equals("thesaurus")) {
                dayStats.setThesaurusCount(val);
            } else if (key.equals("base")) {
                dayStats.setBaseCount(val);
            } else if (key.equals("lexie")) {
                dayStats.setLexieCount(val);
            } else if (key.equals("indexation")) {
                dayStats.setIndexationCount(val);
            } else if (key.equals("version")) {
                version = val;
            }
        }

    }

    private static int getInteger(String s) {
        try {
            return Integer.parseInt(s);
        } catch (NumberFormatException nfe) {
            return -1;
        }
    }


    private class InternalQueryLog extends DefaultQueryLog {

        private void setSearchSequence(String searchSequence) {
            innerSetSearchSequence(searchSequence);
        }

        private void setSite(String site) {
            innerSetSite(site);
        }

        private void setFicheCount(int ficheCount) {
            innerSetFicheCount(ficheCount);
        }

        private void setFicheMaximum(int ficheMaximum) {
            innerSetFicheMaximum(ficheMaximum);
        }

        private void setAllMotcleCount(int allMotcleCount) {
            innerSetAllMotcleCount(allMotcleCount);
        }

        private void setCompleteMotcleCount(int completeMotcleCount) {
            innerSetCompleteMotcleCount(completeMotcleCount);
        }

    }


    private class InternalDayStats implements SearchLog.DayStats {

        private int ficheCount = -1;
        private int motcleCount = -1;
        private int thesaurusCount = -1;
        private int corpusCount = -1;
        private int baseCount = -1;
        private int lexieCount = -1;
        private int indexationCount = -1;

        public int getCorpusCount() {
            return corpusCount;
        }

        public int getThesaurusCount() {
            return thesaurusCount;
        }

        public int getMotcleCount() {
            return motcleCount;
        }

        public int getFicheCount() {
            return ficheCount;
        }

        public int getBaseCount() {
            return baseCount;
        }

        public int getLexieCount() {
            return lexieCount;
        }

        public int getIndexationCount() {
            return indexationCount;
        }

        private void setFicheCount(int ficheCount) {
            this.ficheCount = ficheCount;
        }

        private void setMotcleCount(int motcleCount) {
            this.motcleCount = motcleCount;
        }

        private void setThesaurusCount(int thesaurusCount) {
            this.thesaurusCount = thesaurusCount;
        }

        private void setCorpusCount(int corpusCount) {
            this.corpusCount = corpusCount;
        }

        private void setBaseCount(int baseCount) {
            this.baseCount = baseCount;
        }

        private void setLexieCount(int lexieCount) {
            this.lexieCount = lexieCount;
        }

        private void setIndexationCount(int indexationCount) {
            this.indexationCount = indexationCount;
        }

    }

}
